# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.orm.fedb import FEDB
from hbshare.fe.xwq.analysis.orm.hbdb import HBDB
from hbshare.fe.xwq.analysis.service.industry_analysis_data import get_stock_info
from hbshare.fe.xwq.analysis.utils.const_var import TimeDateFormat
from hbshare.fe.xwq.analysis.utils.timedelta_utils import TimeDateUtil
from datetime import datetime
from scipy import stats
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False
sns.set_style('white', {'font.sans-serif': ['simhei', 'Arial']})
line_color_list = ['#F04950', '#6268A2', '#959595', '#333335', '#EE703F', '#7E4A9B', '#8A662C',
                   '#44488E', '#BA67E9', '#3FAEEE']
bar_color_list = ['#C94649', '#EEB2B4', '#E1777A', '#D57C56', '#E39A79', '#DB8A66', '#E5B88C',
                  '#8588B7', '#B4B6D1', '#55598D', '#628497', '#A9C6CB', '#866EA9', '#B79BC7',
                  '#7D7D7E', '#CACACA', '#A7A7A8', '#606063', '#C4C4C4', '#99999B', '#B7B7B7']
industry_theme_dic = {'银行': '大金融', '非银金融': '大金融', '房地产': '大金融',
                      '食品饮料': '消费', '家用电器': '消费', '医药生物': '消费', '社会服务': '消费', '农林牧渔': '消费', '商贸零售': '消费', '美容护理': '消费',
                      '通信': 'TMT', '计算机': 'TMT', '电子': 'TMT', '传媒': 'TMT', '国防军工': 'TMT',
                      '交通运输': '制造', '机械设备': '制造', '汽车': '制造', '纺织服饰': '制造', '轻工制造': '制造', '电力设备': '制造',
                      '钢铁': '周期', '有色金属': '周期', '建筑装饰': '周期', '建筑材料': '周期', '基础化工': '周期', '石油石化': '周期', '煤炭': '周期', '公用事业': '周期', '环保': '周期',
                      '综合': '其他'}

def get_industry_info():
    industry_info = HBDB().read_industry_info()
    industry_info = industry_info.rename(columns={'flmc': 'INDUSTRY_NAME', 'zsdm': 'INDUSTRY_ID', 'hyhfbz': 'INDUSTRY_VERSION', 'fljb': 'INDUSTRY_TYPE', 'qsrq': 'BEGIN_DATE', 'jsrq': 'END_DATE', 'sfyx': 'IS_NEW'})
    industry_info = industry_info.dropna(subset=['BEGIN_DATE'])
    industry_info['END_DATE'] = industry_info['END_DATE'].replace('', np.nan).fillna('20990101')
    industry_info['BEGIN_DATE'] = industry_info['BEGIN_DATE'].apply(lambda x: datetime.strptime(x, '%Y/%m/%d').strftime('%Y%m%d') if '/' in x else x)
    industry_info['END_DATE'] = industry_info['END_DATE'].apply(lambda x: datetime.strptime(x, '%Y/%m/%d').strftime('%Y%m%d') if '/' in x else x)
    industry_info['BEGIN_DATE'] = industry_info['BEGIN_DATE'].astype(int).astype(str)
    industry_info['END_DATE'] = industry_info['END_DATE'].astype(int).astype(str)
    industry_info['INDUSTRY_VERSION'] = industry_info['INDUSTRY_VERSION'].astype(int)
    industry_info['INDUSTRY_TYPE'] = industry_info['INDUSTRY_TYPE'].astype(int)
    industry_info['IS_NEW'] = industry_info['IS_NEW'].astype(int)
    industry_info = industry_info[industry_info['INDUSTRY_VERSION'] == 3]
    return industry_info

def get_stock_industry():
    stock_industry = HBDB().read_stock_industry()
    stock_industry.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_analysis/stock_industry.hdf', key='table', mode='w')
    stock_industry = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_analysis/stock_industry.hdf', key='table')
    stock_industry = stock_industry.rename(columns={'zqdm': 'TICKER_SYMBOL', 'flmc': 'INDUSTRY_NAME', 'fldm': 'INDUSTRY_ID', 'hyhfbz': 'INDUSTRY_VERSION', 'fljb': 'INDUSTRY_TYPE', 'qsrq': 'BEGIN_DATE', 'jsrq': 'END_DATE', 'sfyx': 'IS_NEW'})
    stock_industry = stock_industry.dropna(subset=['BEGIN_DATE'])
    stock_industry['END_DATE'] = stock_industry['END_DATE'].fillna('20990101')
    stock_industry['BEGIN_DATE'] = stock_industry['BEGIN_DATE'].astype(int).astype(str)
    stock_industry['END_DATE'] = stock_industry['END_DATE'].astype(int).astype(str)
    stock_industry['INDUSTRY_VERSION'] = stock_industry['INDUSTRY_VERSION'].astype(int)
    stock_industry['INDUSTRY_TYPE'] = stock_industry['INDUSTRY_TYPE'].astype(int)
    stock_industry['IS_NEW'] = stock_industry['IS_NEW'].astype(int)
    stock_industry = stock_industry[stock_industry['INDUSTRY_VERSION'] == 2]
    stock_industry = stock_industry.loc[stock_industry['TICKER_SYMBOL'].str.len() == 6]
    stock_industry = stock_industry.loc[stock_industry['TICKER_SYMBOL'].astype(str).str.slice(0, 1).isin(['0', '3', '6'])]
    return stock_industry

class IndustryAnalysis:
    def __init__(self, file_path, start_date, end_date, last_report_date, report_date, con_date, sw_type, select_industry=[]):
        self.file_path = file_path
        self.start_date = start_date
        self.end_date = end_date
        self.sw_type = sw_type
        self.last_report_date = last_report_date
        self.report_date = report_date
        self.con_date = con_date

        if len(select_industry) == 0:
            self.industry_info = get_industry_info()
            self.industry_info = self.industry_info[self.industry_info['INDUSTRY_TYPE'] == self.sw_type]
            self.industry_info = self.industry_info[self.industry_info['IS_NEW'] == 1]
            self.industry_info = self.industry_info[['INDUSTRY_NAME', 'INDUSTRY_ID', 'BEGIN_DATE', 'END_DATE']]
            self.industry_id_name_dic = self.industry_info[['INDUSTRY_ID', 'INDUSTRY_NAME']].set_index('INDUSTRY_ID')['INDUSTRY_NAME'].to_dict()
            self.industry_name_id_dic = self.industry_info[['INDUSTRY_ID', 'INDUSTRY_NAME']].set_index('INDUSTRY_NAME')['INDUSTRY_ID'].to_dict()
            self.select_industry = self.industry_info['INDUSTRY_NAME'].unique().tolist()
        else:
            self.select_industry = select_industry

    def IndustryMarketValue(self):
        ind_mv = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', 'MARKET_VALUE'], 'industry_technology', self.sw_type)
        ind_mv = ind_mv[ind_mv['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_mv['MARKET_VALUE'] = ind_mv['MARKET_VALUE'].apply(lambda x: round(x / 10000000000.0, 2))
        ind_mv = ind_mv.pivot(index='INDUSTRY_NAME', columns='REPORT_DATE', values='MARKET_VALUE')
        date_list = sorted(list(ind_mv.columns))[-12:]
        rank_list = list(ind_mv[max(date_list)].sort_values(ascending=False).index)
        ind_mv = ind_mv.reset_index()
        ind_mv['INDUSTRY_NAME'] = ind_mv['INDUSTRY_NAME'].astype('category')
        ind_mv['INDUSTRY_NAME'].cat.reorder_categories(rank_list, inplace=True)
        ind_mv = ind_mv.sort_values('INDUSTRY_NAME')
        ind_mv = ind_mv.set_index('INDUSTRY_NAME')[date_list]
        #####画热力图#####
        plt.figure(figsize=(12, 8))
        sns.heatmap(ind_mv, annot=True, fmt='.2f', cmap='OrRd')
        plt.xlabel('')
        plt.ylabel('行业市值（百亿）')
        plt.tight_layout()
        plt.savefig('{0}market_value_heatmap.png'.format(self.file_path))
        return

    def IndustryRet(self):
        ind_ret = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', 'RET'], 'industry_technology', self.sw_type)
        ind_ret = ind_ret[ind_ret['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_ret['RET'] = ind_ret['RET'].apply(lambda x: round(x * 100.0, 2))
        ind_ret = ind_ret.pivot(index='INDUSTRY_NAME', columns='REPORT_DATE', values='RET')
        date_list = sorted(list(ind_ret.columns))[-12:]
        rank_list = list(ind_ret[max(date_list)].sort_values(ascending=False).index)
        ind_ret = ind_ret.reset_index()
        ind_ret['INDUSTRY_NAME'] = ind_ret['INDUSTRY_NAME'].astype('category')
        ind_ret['INDUSTRY_NAME'].cat.reorder_categories(rank_list, inplace=True)
        ind_ret = ind_ret.sort_values('INDUSTRY_NAME')
        ind_ret = ind_ret.set_index('INDUSTRY_NAME')[date_list]
        #####画热力图#####
        plt.figure(figsize=(12, 8))
        sns.heatmap(ind_ret, annot=True, fmt='.2f', cmap='OrRd')
        plt.xlabel('')
        plt.ylabel('行业季度涨跌幅（%）')
        plt.tight_layout()
        plt.savefig('{0}ret_heatmap.png'.format(self.file_path))
        return

    def IndustryTech(self):
        ind_tech = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', 'RET', 'VOL', 'BETA', 'ALPHA'], 'industry_technology', self.sw_type)
        ind_tech = ind_tech[ind_tech['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_tech['RET'] = ind_tech['RET'].apply(lambda x: round(x * 100.0, 2))
        ind_tech['VOL'] = ind_tech['VOL'].apply(lambda x: round(x * 100.0, 2))
        ind_tech['BETA'] = ind_tech['BETA'].apply(lambda x: round(x, 2))
        ind_tech['ALPHA'] = ind_tech['ALPHA'].apply(lambda x: round(x * 100.0, 2))
        #####画热力图#####
        fig, ax = plt.subplots(2, 2, figsize=(24, 16))
        tech_list = [['RET', 'VOL'], ['BETA', 'ALPHA']]
        for i in range(2):
            for j in range(2):
                ind_item = ind_tech.pivot(index='INDUSTRY_NAME', columns='REPORT_DATE', values=tech_list[i][j])
                date_list = sorted(list(ind_item.columns))[-12:]
                rank_list = list(ind_item[max(date_list)].sort_values(ascending=False).index)
                ind_item = ind_item.reset_index()
                ind_item['INDUSTRY_NAME'] = ind_item['INDUSTRY_NAME'].astype('category')
                ind_item['INDUSTRY_NAME'].cat.reorder_categories(rank_list, inplace=True)
                ind_item = ind_item.sort_values('INDUSTRY_NAME')
                ind_item = ind_item.set_index('INDUSTRY_NAME')[date_list]
                ylabel = '涨跌幅（%）' if tech_list[i][j] == 'RET' else '波动率（%）' if tech_list[i][j] == 'VOL' else 'BETA' if tech_list[i][j] == 'BETA' else 'ALPHA（%）'
                axij = sns.heatmap(ind_item, ax=ax[i][j], annot=True, fmt='.2f', cmap='OrRd')
                axij.set_xlabel('')
                axij.set_ylabel(ylabel)
        plt.tight_layout()
        plt.savefig('{0}tech_heatmap.png'.format(self.file_path))
        return

    def IndustryNewhigh(self):
        ind_newhigh = pd.read_excel('D:/Git/hbshare/hbshare/fe/xwq/data/new_high/2022年创历史新高个股汇总.xlsx', index_col=0)
        ind_newhigh['股票代码'] = ind_newhigh['股票代码'].apply(lambda x: str(x).zfill(6))
        ind_newhigh['创新高日期'] = ind_newhigh['创新高日期'].apply(lambda x: str(x))
        stock_info = get_stock_info()[['TICKER_SYMBOL', 'SAMPLE_DATE']].rename(columns={'TICKER_SYMBOL': '股票代码', 'SAMPLE_DATE': '入选样本日期'})
        ind_newhigh = ind_newhigh.merge(stock_info, on=['股票代码'], how='inner')
        ind_newhigh = ind_newhigh[ind_newhigh['创新高日期'] >= ind_newhigh['入选样本日期']]
        ind_newhigh = ind_newhigh[(ind_newhigh['创新高日期'] > self.last_report_date) & (ind_newhigh['创新高日期'] <= self.report_date)]
        ind_newhigh = ind_newhigh.sort_values(['股票代码', '创新高日期']).drop_duplicates('股票代码', keep='first')
        ind_newhigh = ind_newhigh.reset_index().drop(['index', '入选样本日期', '创新高日期', '区间内首次创新高', '区间内收盘价最高', '收盘价格'], axis=1)
        ind_newhigh_sw1 = ind_newhigh[['申万一级行业', '股票代码']].groupby('申万一级行业').count().rename(columns={'股票代码': '创新高数量'})
        ind_newhigh_sw1['创新高数量'] = ind_newhigh_sw1['创新高数量'].astype(int)
        ind_newhigh_sw1 = ind_newhigh_sw1.reindex(self.select_industry).fillna(0)
        ind_newhigh_sw1 = ind_newhigh_sw1.reset_index().sort_values('创新高数量', ascending=False)
        #####画柱状图#####
        plt.figure(figsize=(12, 6))
        sns.barplot(x='申万一级行业', y='创新高数量', data=ind_newhigh_sw1, palette=[bar_color_list[0]])
        plt.xlabel('')
        plt.ylabel('创新高个股数量')
        plt.xticks(rotation=90)
        plt.tight_layout()
        plt.savefig('{0}new_high_bar.png'.format(self.file_path))

        fig, ax = plt.subplots(2, 2, figsize=(12, 8))
        ind_newhigh_top4 = ind_newhigh_sw1['申万一级行业'].unique().tolist()[:4]
        new_high_list = [[ind_newhigh_top4[0], ind_newhigh_top4[1]], [ind_newhigh_top4[2], ind_newhigh_top4[3]]]
        for i in range(2):
            for j in range(2):
                ind_newhigh_sw2 = ind_newhigh[ind_newhigh['申万一级行业'] == new_high_list[i][j]]
                ind_newhigh_sw2 = ind_newhigh_sw2[['申万二级行业', '股票代码']].groupby('申万二级行业').count().rename(columns={'股票代码': '创新高数量'})
                ind_newhigh_sw2['创新高数量'] = ind_newhigh_sw2['创新高数量'].astype(int)
                ind_newhigh_sw2 = ind_newhigh_sw2.reset_index().sort_values('创新高数量', ascending=False)
                ax[i][j].pie(ind_newhigh_sw2['创新高数量'].values, labels=ind_newhigh_sw2['申万二级行业'].values, autopct= '%0.2f%%', colors=line_color_list)
                ax[i][j].set_xlabel(new_high_list[i][j])
                ax[i][j].set_ylabel('')
        plt.tight_layout()
        plt.savefig('{0}new_high_pie.png'.format(self.file_path))
        return

    def IndustryVal(self):
        ind_pe = FEDB().read_industry_data(['TRADE_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', 'PE_TTM'], 'industry_daily_valuation', self.sw_type)
        ind_pe = ind_pe[ind_pe['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_pe_latest = ind_pe[ind_pe['TRADE_DATE'] == ind_pe['TRADE_DATE'].max()]
        ind_pe_min = ind_pe[['INDUSTRY_NAME', 'PE_TTM']].groupby('INDUSTRY_NAME').min().reset_index().rename(columns={'PE_TTM': 'PE_TTM_MIN'})
        ind_pe_mean = ind_pe[['INDUSTRY_NAME', 'PE_TTM']].groupby('INDUSTRY_NAME').mean().reset_index().rename(columns={'PE_TTM': 'PE_TTM_MEAN'})
        ind_pe_median = ind_pe[['INDUSTRY_NAME', 'PE_TTM']].groupby('INDUSTRY_NAME').median().reset_index().rename(columns={'PE_TTM': 'PE_TTM_MEDIAN'})
        ind_pe_max = ind_pe[['INDUSTRY_NAME', 'PE_TTM']].groupby('INDUSTRY_NAME').max().reset_index().rename(columns={'PE_TTM': 'PE_TTM_MAX'})
        ind_pe_quantile = ind_pe[['INDUSTRY_NAME', 'TRADE_DATE', 'PE_TTM']].groupby('INDUSTRY_NAME').apply(lambda x: (1.0 - np.count_nonzero(x.sort_values('TRADE_DATE')['PE_TTM'].iloc[-1] <= x['PE_TTM']) / x['PE_TTM'].size) * 100.0)
        ind_pe_quantile = pd.DataFrame(ind_pe_quantile).reset_index().rename(columns={0: 'PE_TTM_QUANTILE'})
        ind_pe_disp = ind_pe_latest.merge(ind_pe_min, on=['INDUSTRY_NAME'], how='left') \
                                   .merge(ind_pe_mean, on=['INDUSTRY_NAME'], how='left') \
                                   .merge(ind_pe_median, on=['INDUSTRY_NAME'], how='left') \
                                   .merge(ind_pe_max, on=['INDUSTRY_NAME'], how='left') \
                                   .merge(ind_pe_quantile, on=['INDUSTRY_NAME'], how='left')
        ind_pe_disp = ind_pe_disp.sort_values('PE_TTM_QUANTILE')
        for col in ['PE_TTM', 'PE_TTM_QUANTILE', 'PE_TTM_MIN', 'PE_TTM_MEAN', 'PE_TTM_MEDIAN', 'PE_TTM_MAX']:
            ind_pe_disp[col] = ind_pe_disp[col].apply(lambda x: round(x, 2))
        ind_pe_disp['PE_TTM_QUANTILE'] = ind_pe_disp['PE_TTM_QUANTILE'].apply(lambda x: '{0}%'.format(x))
        ind_pe_disp = ind_pe_disp[['INDUSTRY_NAME', 'PE_TTM_QUANTILE', 'PE_TTM', 'PE_TTM_MIN', 'PE_TTM_MEAN', 'PE_TTM_MEDIAN', 'PE_TTM_MAX']]
        ind_pe_disp.columns = ['行业名称', '分位水平', 'PE_TTM', '最小值', '平均值', '中位数', '最大值']
        ind_pe_disp.to_excel('{0}pe_ttm_disp.xlsx'.format(self.file_path))
        ind_pe = ind_pe.pivot(index='TRADE_DATE', columns='INDUSTRY_NAME', values='PE_TTM').sort_index()
        ind_list = ind_pe_disp['行业名称'].unique().tolist()[::-1]
        ind_pe_list = [ind_pe[ind] for ind in ind_list]
        ind_pe_latest['INDUSTRY_NAME'] = ind_pe_latest['INDUSTRY_NAME'].astype('category')
        ind_pe_latest['INDUSTRY_NAME'].cat.reorder_categories(ind_list, inplace=True)
        ind_pe_latest = ind_pe_latest.sort_values('INDUSTRY_NAME')
        plt.figure(figsize=(12, 12))
        plt.boxplot(ind_pe_list, labels=ind_list, vert=False, flierprops={'marker': 'o', 'markersize': 1}, meanline=True, showmeans=True)
        plt.scatter(ind_pe_latest['PE_TTM'], range(1, len(ind_pe_latest) + 1), marker='o')
        plt.tight_layout()
        plt.savefig('{0}pe_ttm_box.png'.format(self.file_path))

        rank_list = [self.report_date, self.last_report_date]
        ind_pe = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', 'PE_TTM'], 'industry_valuation', self.sw_type)
        ind_pe = ind_pe[ind_pe['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_pe = ind_pe[ind_pe['REPORT_DATE'].isin(rank_list)]
        ind_pe['PE_TTM'] = ind_pe['PE_TTM'].apply(lambda x: round(x, 2))
        ind_pe = ind_pe.sort_values(['REPORT_DATE', 'PE_TTM'], ascending=[False, False])
        ind_pe = ind_pe[['REPORT_DATE', 'INDUSTRY_NAME', 'PE_TTM']]
        ind_pe.columns = ['报告日期', '行业名称', 'PE_TTM']
        #####画柱状图#####
        plt.figure(figsize=(12, 6))
        sns.barplot(x='行业名称', y='PE_TTM', data=ind_pe, hue='报告日期', hue_order=rank_list, palette=[bar_color_list[0], bar_color_list[7]])
        plt.xlabel('')
        plt.ylabel('PE_TTM')
        plt.xticks(rotation=90)
        plt.legend(loc=1)
        plt.tight_layout()
        plt.savefig('{0}pe_ttm_bar.png'.format(self.file_path))

        rank_list = [self.report_date, self.last_report_date]
        ind_pb = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', 'PB_LF'], 'industry_valuation', self.sw_type)
        ind_pb = ind_pb[ind_pb['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_pb = ind_pb[ind_pb['REPORT_DATE'].isin(rank_list)]
        ind_pb['PB_LF'] = ind_pb['PB_LF'].apply(lambda x: round(x, 2))
        ind_pb = ind_pb.sort_values(['REPORT_DATE', 'PB_LF'], ascending=[False, False])
        ind_pb = ind_pb[['REPORT_DATE', 'INDUSTRY_NAME', 'PB_LF']]
        ind_pb.columns = ['报告日期', '行业名称', 'PB_LF']
        #####画柱状图#####
        plt.figure(figsize=(12, 6))
        sns.barplot(x='行业名称', y='PB_LF', data=ind_pb, hue='报告日期', hue_order=rank_list, palette=[bar_color_list[0], bar_color_list[7]])
        plt.xlabel('')
        plt.ylabel('PB_LF')
        plt.xticks(rotation=90)
        plt.legend(loc=1)
        plt.tight_layout()
        plt.savefig('{0}pb_lf_bar.png'.format(self.file_path))

        ind_val = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', 'PE_TTM', 'PB_LF'], 'industry_valuation', self.sw_type)
        ind_val = ind_val[ind_val['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_val['PE_TTM'] = ind_val['PE_TTM'].apply(lambda x: round(x, 2))
        ind_val['PB_LF'] = ind_val['PB_LF'].apply(lambda x: round(x, 2))
        #####画热力图#####
        fig, ax = plt.subplots(1, 2, figsize=(24, 8))
        val_list = ['PE_TTM', 'PB_LF']
        for i in range(2):
            ind_item = ind_val.pivot(index='INDUSTRY_NAME', columns='REPORT_DATE', values=val_list[i])
            date_list = sorted(list(ind_item.columns))[-12:]
            rank_list = list(ind_item[max(date_list)].sort_values(ascending=False).index)
            ind_item = ind_item.reset_index()
            ind_item['INDUSTRY_NAME'] = ind_item['INDUSTRY_NAME'].astype('category')
            ind_item['INDUSTRY_NAME'].cat.reorder_categories(rank_list, inplace=True)
            ind_item = ind_item.sort_values('INDUSTRY_NAME')
            ind_item = ind_item.set_index('INDUSTRY_NAME')[date_list]
            axi = sns.heatmap(ind_item, ax=ax[i], annot=True, fmt='.2f', cmap='OrRd')
            axi.set_xlabel('')
            axi.set_ylabel(val_list[i])
        plt.tight_layout()
        plt.savefig('{0}val_heatmap.png'.format(self.file_path))
        return

    def IndustryFmt(self):
        date_list = ['20191231', '20201231', '20210331', '20210630', '20210930', '20211231', '20220331']
        ind_fmt = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', 'NET_PROFIT_ACCUM_YOY'], 'industry_fundamental_derive', self.sw_type)
        ind_fmt = ind_fmt[ind_fmt['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_fmt = ind_fmt[ind_fmt['REPORT_DATE'].isin(date_list)]
        ind_fmt = ind_fmt.pivot(index='INDUSTRY_NAME', columns='REPORT_DATE', values='NET_PROFIT_ACCUM_YOY').reset_index()
        ind_fmt['THEME'] = ind_fmt['INDUSTRY_NAME'].apply(lambda x: industry_theme_dic[x])
        ind_fmt['THEME'] = ind_fmt['THEME'].astype('category')
        ind_fmt['THEME'].cat.reorder_categories(['周期', '制造', '消费', '大金融', 'TMT', '其他'], inplace=True)
        ind_fmt = ind_fmt.sort_values(['THEME', self.report_date], ascending=[True, False])
        for date in date_list:
            ind_fmt[date] = ind_fmt[date].apply(lambda x: '{0}%'.format(round(x * 100.0, 2)))
        ind_fmt = ind_fmt[['THEME', 'INDUSTRY_NAME'] + date_list]
        ind_fmt.columns = ['主题', '行业', '2019A', '2020A', '2021Q1', '2021Q2', '2021Q3', '2021A', '2022Q1']
        ind_fmt.to_excel('{0}net_profit_accum_yoy_disp.xlsx'.format(self.file_path))

        for index_name in ['ROE_TTM', 'GROSS_INCOME_RATIO_TTM']:
            ylabel = 'ROE_TTM' if index_name == 'ROE_TTM' else '毛利率_TTM'

            rank_list = [self.report_date, self.last_report_date]
            ind_fmt = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', index_name], 'industry_fundamental', self.sw_type)
            ind_fmt = ind_fmt[ind_fmt['INDUSTRY_NAME'].isin(self.select_industry)]
            ind_fmt = ind_fmt[ind_fmt['REPORT_DATE'].isin(rank_list)]
            ind_fmt[index_name] = ind_fmt[index_name].apply(lambda x: round(x, 2))
            ind_fmt = ind_fmt.sort_values(['REPORT_DATE', index_name], ascending=[False, False])
            ind_fmt = ind_fmt[['REPORT_DATE', 'INDUSTRY_NAME', index_name]]
            ind_fmt.columns = ['报告日期', '行业名称', index_name]
            #####画柱状图#####
            plt.figure(figsize=(12, 6))
            sns.barplot(x='行业名称', y=index_name, data=ind_fmt, hue='报告日期', hue_order=rank_list, palette=[bar_color_list[0], bar_color_list[7]])
            plt.xlabel('')
            plt.ylabel(ylabel + '（%）')
            plt.xticks(rotation=90)
            plt.legend(loc=1)
            plt.tight_layout()
            plt.savefig('{0}{1}_bar.png'.format(self.file_path, index_name.lower()))

            industry_list = ind_fmt[ind_fmt['报告日期'] == self.report_date]['行业名称'].unique().tolist()
            mom_abs = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', index_name + '_MOM_ABS'], 'industry_fundamental_derive', self.sw_type)
            mom_abs = mom_abs[mom_abs['INDUSTRY_NAME'].isin(self.select_industry)]
            mom_abs = mom_abs[mom_abs['REPORT_DATE'].isin([self.report_date])]
            mom_abs[index_name + '_MOM_ABS'] = mom_abs[index_name + '_MOM_ABS'].apply(lambda x: round(x, 2))
            mom_abs['INDUSTRY_NAME'] = mom_abs['INDUSTRY_NAME'].astype('category')
            mom_abs['INDUSTRY_NAME'].cat.reorder_categories(industry_list, inplace=True)
            mom_abs = mom_abs.sort_values('INDUSTRY_NAME')
            mom_abs = mom_abs[['REPORT_DATE', 'INDUSTRY_NAME', index_name + '_MOM_ABS']]
            mom_abs.columns = ['报告日期', '行业名称', index_name + '_MOM_ABS']
            #####画柱状图#####
            plt.figure(figsize=(12, 6))
            sns.barplot(x='行业名称', y=index_name + '_MOM_ABS', data=mom_abs, palette=[bar_color_list[0]])
            plt.xlabel('')
            plt.ylabel(ylabel + '环比变化（%）')
            plt.xticks(rotation=90)
            plt.tight_layout()
            plt.savefig('{0}{1}_mom_abs_bar.png'.format(self.file_path, index_name.lower()))

            yoy = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', index_name + '_YOY'], 'industry_fundamental_derive', self.sw_type)
            yoy = yoy[yoy['INDUSTRY_NAME'].isin(self.select_industry)]
            yoy = yoy[yoy['REPORT_DATE'].isin(rank_list)]
            yoy[index_name + '_YOY'] = yoy[index_name + '_YOY'].apply(lambda x: round(x * 100.0, 2))
            yoy = yoy.sort_values(['REPORT_DATE', index_name + '_YOY'], ascending=[False, False])
            yoy = yoy[['REPORT_DATE', 'INDUSTRY_NAME', index_name + '_YOY']]
            yoy.columns = ['报告日期', '行业名称', index_name + '_YOY']
            #####画柱状图#####
            plt.figure(figsize=(12, 6))
            sns.barplot(x='行业名称', y=index_name + '_YOY', data=yoy, hue='报告日期', hue_order=rank_list, palette=[bar_color_list[0], bar_color_list[7]])
            plt.xlabel('')
            plt.ylabel(ylabel + '同比增速（%）')
            plt.xticks(rotation=90)
            plt.legend(loc=1)
            plt.tight_layout()
            plt.savefig('{0}{1}_yoy_bar.png'.format(self.file_path, index_name.lower()))
        return

    def IndustryCon(self):
        ind_con_sw1 = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', 'EXCEED_RATIO_NEW'], 'industry_consensus', self.sw_type)
        ind_con_sw1 = ind_con_sw1[ind_con_sw1['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_con_sw1 = ind_con_sw1[ind_con_sw1['REPORT_DATE'] == self.con_date]
        ind_con_sw1['EXCEED_RATIO_NEW'] = ind_con_sw1['EXCEED_RATIO_NEW'].apply(lambda x: round(x * 100.0, 2))
        ind_con_sw1 = ind_con_sw1.set_index('INDUSTRY_NAME').reindex(self.select_industry).fillna(0)
        ind_con_sw1 = ind_con_sw1.reset_index().sort_values('EXCEED_RATIO_NEW', ascending=False)
        ind_con_sw1 = ind_con_sw1[['REPORT_DATE', 'INDUSTRY_NAME', 'EXCEED_RATIO_NEW']]
        ind_con_sw1.columns = ['报告日期', '行业名称', '超预期个股占比（%）']
        #####画柱状图#####
        plt.figure(figsize=(12, 6))
        sns.barplot(x='行业名称', y='超预期个股占比（%）', data=ind_con_sw1, palette=[bar_color_list[0]])
        plt.xlabel('')
        plt.ylabel('超预期个股占比（%）')
        plt.xticks(rotation=90)
        plt.tight_layout()
        plt.savefig('{0}exceed_ratio_bar.png'.format(self.file_path))

        ind_con_sw2 = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', 'EXCEED_RATIO_NEW'], 'industry_consensus', 2)
        stock_industry = get_stock_industry()
        stock_industry_sw1 = stock_industry[stock_industry['INDUSTRY_TYPE'] == 1].rename(columns={'INDUSTRY_NAME': 'INDUSTRY_NAME_SW1'})
        stock_industry_sw2 = stock_industry[stock_industry['INDUSTRY_TYPE'] == 2].rename(columns={'INDUSTRY_NAME': 'INDUSTRY_NAME_SW2'})
        stock_industry = stock_industry_sw1[['TICKER_SYMBOL', 'INDUSTRY_NAME_SW1']].drop_duplicates().merge(stock_industry_sw2[['TICKER_SYMBOL', 'INDUSTRY_NAME_SW2']].drop_duplicates(), on=['TICKER_SYMBOL'], how='left')
        fig, ax = plt.subplots(2, 2, figsize=(18, 12))
        ind_con_top4 = ind_con_sw1['行业名称'].unique().tolist()[:4]
        ind_con_list = [[ind_con_top4[0], ind_con_top4[1]], [ind_con_top4[2], ind_con_top4[3]]]
        for i in range(2):
            for j in range(2):
                ind_sw2 = ind_con_sw2[ind_con_sw2['INDUSTRY_NAME'].isin(stock_industry[stock_industry['INDUSTRY_NAME_SW1'] == ind_con_list[i][j]]['INDUSTRY_NAME_SW2'].unique().tolist())]
                ind_sw2 = ind_sw2[ind_sw2['REPORT_DATE'] == self.con_date]
                ind_sw2['EXCEED_RATIO_NEW'] = ind_sw2['EXCEED_RATIO_NEW'].apply(lambda x: round(x * 100.0, 2))
                ind_sw2 = ind_sw2.sort_values('EXCEED_RATIO_NEW', ascending=False)
                ind_sw2 = ind_sw2[['REPORT_DATE', 'INDUSTRY_NAME', 'EXCEED_RATIO_NEW']]
                ind_sw2.columns = ['报告日期', '行业名称', '超预期个股占比（%）']
                ax[i][j].pie(ind_sw2['超预期个股占比（%）'].values, labels=ind_sw2['行业名称'].values, autopct='%0.2f%%', colors=line_color_list)
                ax[i][j].set_xlabel(ind_con_list[i][j])
                ax[i][j].set_ylabel('')
        plt.tight_layout()
        plt.savefig('{0}exceed_ratio_pie.png'.format(self.file_path))

        index_name = 'EST_ROE_FY1'
        rank_list = [self.report_date, self.last_report_date]
        ind_con = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', index_name], 'industry_consensus', self.sw_type)
        ind_con = ind_con[ind_con['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_con = ind_con[ind_con['REPORT_DATE'].isin(rank_list)]
        ind_con[index_name] = ind_con[index_name].apply(lambda x: round(x, 2))
        ind_con = ind_con.sort_values(['REPORT_DATE', index_name], ascending=[False, False])
        ind_con = ind_con[['REPORT_DATE', 'INDUSTRY_NAME', index_name]]
        ind_con.columns = ['报告日期', '行业名称', '一致预期ROE（%）']
        industry_list = ind_con[ind_con['报告日期'] == self.report_date]['行业名称'].unique().tolist()
        mom_abs = ind_con[ind_con['报告日期'] == self.report_date][['行业名称', '一致预期ROE（%）']].set_index('行业名称') - ind_con[ind_con['报告日期'] == self.last_report_date][['行业名称', '一致预期ROE（%）']].set_index('行业名称')
        mom_abs = pd.DataFrame(mom_abs).rename(columns={'一致预期ROE（%）': '一致预期ROE环比变化（%）'}).reset_index()
        mom_abs['行业名称'] = mom_abs['行业名称'].astype('category')
        mom_abs['行业名称'].cat.reorder_categories(industry_list, inplace=True)
        mom_abs = mom_abs.sort_values('行业名称')
        #####画柱状图#####
        plt.figure(figsize=(12, 6))
        sns.barplot(x='行业名称', y='一致预期ROE（%）', data=ind_con, hue='报告日期', hue_order=rank_list, palette=[bar_color_list[0], bar_color_list[7]])
        plt.xlabel('')
        plt.ylabel('一致预期ROE（%）')
        plt.xticks(rotation=90)
        plt.legend(loc=1)
        plt.tight_layout()
        plt.savefig('{0}{1}_bar.png'.format(self.file_path, index_name.lower()))
        plt.figure(figsize=(12, 6))
        sns.barplot(x='行业名称', y='一致预期ROE环比变化（%）', data=mom_abs, palette=[bar_color_list[0]])
        plt.xlabel('')
        plt.ylabel('一致预期ROE环比变化（%）')
        plt.xticks(rotation=90)
        plt.tight_layout()
        plt.savefig('{0}{1}_mom_abs_bar.png'.format(self.file_path, index_name.lower()))

        index_name = 'EST_ROE_YOY'
        rank_list = [self.report_date, self.last_report_date]
        ind_con = FEDB().read_industry_data(['REPORT_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', index_name], 'industry_consensus', self.sw_type)
        ind_con = ind_con[ind_con['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_con = ind_con[ind_con['REPORT_DATE'].isin(rank_list)]
        ind_con[index_name] = ind_con[index_name].apply(lambda x: round(x, 2))
        ind_con = ind_con.sort_values(['REPORT_DATE', index_name], ascending=[False, False])
        ind_con = ind_con[['REPORT_DATE', 'INDUSTRY_NAME', index_name]]
        ind_con.columns = ['报告日期', '行业名称', '一致预期ROE同比增速（%）']
        #####画柱状图#####
        plt.figure(figsize=(12, 6))
        sns.barplot(x='行业名称', y='一致预期ROE同比增速（%）', data=ind_con, hue='报告日期', hue_order=rank_list, palette=[bar_color_list[0], bar_color_list[7]])
        plt.xlabel('')
        plt.ylabel('一致预期ROE同比增速（%）')
        plt.xticks(rotation=90)
        plt.legend(loc=1)
        plt.tight_layout()
        plt.savefig('{0}{1}_bar.png'.format(self.file_path, index_name.lower()))
        return

    def IndustryCrowding(self):
        rank_list = [self.report_date, self.last_report_date]
        ind_daily_crowding = FEDB().read_industry_data(['TRADE_DATE', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'INDUSTRY_TYPE', 'VOL', 'TURNOVER', 'BETA'], 'industry_daily_technology', self.sw_type)
        ind_daily_crowding = ind_daily_crowding[ind_daily_crowding['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_daily_crowding['VOL'] = ind_daily_crowding['VOL'].rolling(60).mean()
        ind_daily_crowding['TURNOVER'] = ind_daily_crowding['TURNOVER'].rolling(60).mean()
        ind_daily_crowding['BETA'] = ind_daily_crowding['BETA'].rolling(60).mean()
        ind_daily_crowding_date = ind_daily_crowding[ind_daily_crowding['TRADE_DATE'] <= self.report_date]
        ind_daily_vol_quantile_date = ind_daily_crowding_date[['INDUSTRY_NAME', 'TRADE_DATE', 'VOL']].groupby('INDUSTRY_NAME').apply(lambda x: stats.percentileofscore(x['VOL'], x.sort_values('TRADE_DATE')['VOL'].iloc[-1]))
        ind_daily_turnover_quantile_date = ind_daily_crowding_date[['INDUSTRY_NAME', 'TRADE_DATE', 'TURNOVER']].groupby('INDUSTRY_NAME').apply(lambda x: stats.percentileofscore(x['TURNOVER'], x.sort_values('TRADE_DATE')['TURNOVER'].iloc[-1]))
        ind_daily_beta_quantile_date = ind_daily_crowding_date[['INDUSTRY_NAME', 'TRADE_DATE', 'BETA']].groupby('INDUSTRY_NAME').apply(lambda x: stats.percentileofscore(x['BETA'], x.sort_values('TRADE_DATE')['BETA'].iloc[-1]))
        ind_crowding_date = (ind_daily_vol_quantile_date + ind_daily_turnover_quantile_date + ind_daily_beta_quantile_date) / 3.0
        ind_crowding_date = pd.DataFrame(ind_crowding_date).reset_index()
        ind_crowding_date.columns = ['行业名称', '行业拥挤度（%）']
        ind_crowding_date['报告日期'] = self.report_date
        ind_crowding_date = ind_crowding_date.sort_values('行业拥挤度（%）', ascending=False)
        industry_list = ind_crowding_date['行业名称'].unique().tolist()
        ind_daily_crowding_last_date = ind_daily_crowding[ind_daily_crowding['TRADE_DATE'] <= self.last_report_date]
        ind_daily_vol_quantile_last_date = ind_daily_crowding_last_date[['INDUSTRY_NAME', 'TRADE_DATE', 'VOL']].groupby('INDUSTRY_NAME').apply(lambda x: stats.percentileofscore(x['VOL'], x.sort_values('TRADE_DATE')['VOL'].iloc[-1]))
        ind_daily_turnover_quantile_last_date = ind_daily_crowding_last_date[['INDUSTRY_NAME', 'TRADE_DATE', 'TURNOVER']].groupby('INDUSTRY_NAME').apply(lambda x: stats.percentileofscore(x['TURNOVER'], x.sort_values('TRADE_DATE')['TURNOVER'].iloc[-1]))
        ind_daily_beta_quantile_last_date = ind_daily_crowding_last_date[['INDUSTRY_NAME', 'TRADE_DATE', 'BETA']].groupby('INDUSTRY_NAME').apply(lambda x: stats.percentileofscore(x['BETA'], x.sort_values('TRADE_DATE')['BETA'].iloc[-1]))
        ind_crowding_last_date = (ind_daily_vol_quantile_last_date + ind_daily_turnover_quantile_last_date + ind_daily_beta_quantile_last_date) / 3.0
        ind_crowding_last_date = pd.DataFrame(ind_crowding_last_date).reset_index()
        ind_crowding_last_date.columns = ['行业名称', '行业拥挤度（%）']
        ind_crowding_last_date['报告日期'] = self.last_report_date
        ind_crowding_last_date = ind_crowding_last_date.sort_values('行业拥挤度（%）', ascending=False)
        ind_crowding = pd.concat([ind_crowding_date, ind_crowding_last_date])
        ind_crowding = ind_crowding.sort_values(['报告日期', '行业拥挤度（%）'], ascending=[False, False])
        mom_abs = ind_crowding_date[['行业名称', '行业拥挤度（%）']].set_index('行业名称') - ind_crowding_last_date[['行业名称', '行业拥挤度（%）']].set_index('行业名称')
        mom_abs = pd.DataFrame(mom_abs).rename(columns={'行业拥挤度（%）': '行业拥挤度环比变化（%）'}).reset_index()
        mom_abs['行业名称'] = mom_abs['行业名称'].astype('category')
        mom_abs['行业名称'].cat.reorder_categories(industry_list, inplace=True)
        mom_abs = mom_abs.sort_values('行业名称')
        #####画柱状图#####
        plt.figure(figsize=(12, 6))
        sns.barplot(x='行业名称', y='行业拥挤度（%）', data=ind_crowding, hue='报告日期', hue_order=rank_list, palette=[bar_color_list[0], bar_color_list[7]])
        plt.xlabel('')
        plt.ylabel('行业拥挤度（%）')
        plt.xticks(rotation=90)
        plt.legend(loc=1)
        plt.tight_layout()
        plt.savefig('{0}ind_crowding_bar.png'.format(self.file_path))
        plt.figure(figsize=(12, 6))
        sns.barplot(x='行业名称', y='行业拥挤度环比变化（%）', data=mom_abs, palette=[bar_color_list[0]])
        plt.xlabel('')
        plt.ylabel('行业拥挤度环比变化（%）')
        plt.xticks(rotation=90)
        plt.tight_layout()
        plt.savefig('{0}ind_crowding_mom_abs_bar.png'.format(self.file_path))
        return

    def IndustryHolding(self):
        # 正常运行中的普通股票型、偏股混合型、灵活配置型公募基金
        fund = HBDB().read_stock_fund_info()
        fund = fund.rename(columns={'jjdm': 'FUND_CODE', 'jjmc': 'FUND_FULL_NAME', 'jjjc': 'FUND_SHORT_NAME', 'clrq': 'BEGIN_DATE', 'zzrq': 'END_DATE', 'ejfl': 'FUND_TYPE', 'kffb': 'OPEN_CLOSE'})
        fund['END_DATE'] = fund['END_DATE'].fillna(20990101)
        fund['BEGIN_DATE'] = fund['BEGIN_DATE'].astype(str)
        fund['END_DATE'] = fund['END_DATE'].astype(str)
        # 成立距计算日期满2年
        date_before = TimeDateUtil.get_previous_date_str(self.report_date, TimeDateFormat.YMD.value, TimeDateFormat.YMD.value, 730)
        fund = fund[(fund['BEGIN_DATE'] <= date_before) & (fund['END_DATE'] >= self.report_date)]
        fund = fund.sort_values(['FUND_FULL_NAME', 'FUND_CODE']).drop_duplicates('FUND_FULL_NAME')
        # 成立以来股票占基金净资产的比例均值不低于60%
        fund_gptzzjb = HBDB().read_fund_gptzzjb_given_codes(fund['FUND_CODE'].unique().tolist())
        fund_gptzzjb = fund_gptzzjb.rename(columns={'jjdm': 'FUND_CODE', 'jsrq': 'REPORT_DATE', 'gptzzjb': 'EQUITY_IN_NA'})
        fund_gptzzjb['REPORT_DATE'] = fund_gptzzjb['REPORT_DATE'].astype(str)
        fund_gptzzjb_mean = fund_gptzzjb[['FUND_CODE', 'EQUITY_IN_NA']].groupby('FUND_CODE').mean().reset_index()
        fund_gptzzjb_mean = fund_gptzzjb_mean[fund_gptzzjb_mean['EQUITY_IN_NA'] >= 60]
        fund = fund[fund['FUND_CODE'].isin(fund_gptzzjb_mean['FUND_CODE'].unique().tolist())]
        # 近2年以来股票占基金净资产的比例均不低于50%
        fund_gptzzjb = fund_gptzzjb[(fund_gptzzjb['REPORT_DATE'] >= date_before) & (fund_gptzzjb['REPORT_DATE'] <= self.report_date)]
        fund_gptzzjb_min = fund_gptzzjb[['FUND_CODE', 'EQUITY_IN_NA']].groupby('FUND_CODE').min().reset_index()
        fund_gptzzjb_min = fund_gptzzjb_min[fund_gptzzjb_min['EQUITY_IN_NA'] >= 50]
        fund = fund[fund['FUND_CODE'].isin(fund_gptzzjb_min['FUND_CODE'].unique().tolist())]
        # 基金持仓
        fund_holding = HBDB().read_fund_holding_given_codes(fund['FUND_CODE'].unique().tolist())
        fund_holding.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/mutual_analysis/fund_holding.hdf', key='table', mode='w')
        fund_holding = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/mutual_analysis/fund_holding.hdf', key='table')
        fund_holding = fund_holding.rename(columns={'jjdm': 'FUND_CODE', 'jsrq': 'REPORT_DATE', 'zqdm': 'TICKER_SYMBOL', 'zqmc': 'SEC_SHORT_NAME', 'ccsz': 'HOLDING_MARKET_VALUE', 'ccsl': 'HOLDING_AMOUNT', 'zjbl': 'MV_IN_NA'})
        fund_holding['REPORT_DATE'] = fund_holding['REPORT_DATE'].astype(str)
        # 基金重仓
        fund_zc_holding = fund_holding.sort_values(['FUND_CODE', 'REPORT_DATE', 'MV_IN_NA'], ascending=[True, True, False]).groupby(['FUND_CODE', 'REPORT_DATE']).head(10)
        # 股票行业对应关系
        stock_industry = get_stock_industry()
        stock_industry = stock_industry[stock_industry['INDUSTRY_TYPE'] == self.sw_type]
        stock_industry = stock_industry[stock_industry['IS_NEW'] == 1]
        stock_industry = stock_industry[['INDUSTRY_NAME', 'TICKER_SYMBOL']]
        fund_zc_holding_industry = fund_zc_holding.merge(stock_industry, on=['TICKER_SYMBOL'], how='left')
        fund_zc_holding_industry = fund_zc_holding_industry[['REPORT_DATE', 'FUND_CODE', 'INDUSTRY_NAME', 'MV_IN_NA']].groupby(['REPORT_DATE', 'FUND_CODE', 'INDUSTRY_NAME']).sum().reset_index()
        fund_zc_holding_industry = fund_zc_holding_industry[['REPORT_DATE', 'INDUSTRY_NAME', 'MV_IN_NA']].groupby(['REPORT_DATE', 'INDUSTRY_NAME']).mean().reset_index()

        rank_list = [self.report_date, self.last_report_date]
        ind_holding = fund_zc_holding_industry[fund_zc_holding_industry['INDUSTRY_NAME'].isin(self.select_industry)]
        ind_holding = ind_holding[ind_holding['INDUSTRY_NAME'] != '综合']
        ind_holding = ind_holding[ind_holding['REPORT_DATE'].isin(rank_list)]
        ind_holding['MV_IN_NA'] = ind_holding['MV_IN_NA'].apply(lambda x: round(x, 2))
        ind_holding_list = []
        for date in rank_list:
            ind_holding_date = ind_holding[ind_holding['REPORT_DATE'] == date][['INDUSTRY_NAME', 'MV_IN_NA']]
            ind_holding_date = ind_holding_date.set_index('INDUSTRY_NAME').reindex(self.select_industry).fillna(0).reset_index()
            ind_holding_date['REPORT_DATE'] = date
            ind_holding_list.append(ind_holding_date)
        ind_holding = pd.concat(ind_holding_list)
        ind_holding = ind_holding.sort_values(['REPORT_DATE', 'MV_IN_NA'], ascending=[False, False])
        ind_holding = ind_holding[['INDUSTRY_NAME', 'REPORT_DATE', 'MV_IN_NA']]
        ind_holding.columns = ['行业名称', '报告日期', '持仓比例（%）']
        industry_list = ind_holding[ind_holding['报告日期'] == self.report_date]['行业名称'].unique().tolist()
        mom_abs = ind_holding[ind_holding['报告日期'] == self.report_date][['行业名称', '持仓比例（%）']].set_index('行业名称') - ind_holding[ind_holding['报告日期'] == self.last_report_date][['行业名称', '持仓比例（%）']].set_index('行业名称')
        mom_abs = pd.DataFrame(mom_abs).rename(columns={'持仓比例（%）': '持仓比例环比变化（%）'}).reset_index()
        mom_abs['行业名称'] = mom_abs['行业名称'].astype('category')
        mom_abs['行业名称'].cat.reorder_categories(industry_list, inplace=True)
        mom_abs = mom_abs.sort_values('行业名称')
        #####画柱状图#####
        plt.figure(figsize=(12, 6))
        sns.barplot(x='行业名称', y='持仓比例（%）', data=ind_holding, hue='报告日期', hue_order=rank_list, palette=[bar_color_list[0], bar_color_list[7]])
        plt.xlabel('')
        plt.ylabel('持仓比例（%）')
        plt.legend(loc=1)
        plt.xticks(rotation=90)
        plt.tight_layout()
        plt.savefig('{0}ind_holding_bar.png'.format(self.file_path))
        plt.figure(figsize=(12, 6))
        sns.barplot(x='行业名称', y='持仓比例环比变化（%）', data=mom_abs, palette=[bar_color_list[0]])
        plt.xlabel('')
        plt.ylabel('持仓比例环比变化（%）')
        plt.xticks(rotation=90)
        plt.tight_layout()
        plt.savefig('{0}ind_holding_mom_abs_bar.png'.format(self.file_path))
        return

    def get_all(self):
        self.IndustryMarketValue()
        self.IndustryRet()
        self.IndustryTech()
        self.IndustryNewhigh()
        self.IndustryVal()
        self.IndustryFmt()
        self.IndustryCon()
        self.IndustryCrowding()
        self.IndustryHolding()
        return


if __name__ == '__main__':
    file_path = 'D:/Git/hbshare/hbshare/fe/xwq/data/industry_analysis/'
    start_date = '20170101'
    end_date = '20220331'
    last_report_date = '20211231'
    report_date = '20220331'
    con_date = '20211231'
    sw_type = 1
    IndustryAnalysis(file_path, start_date, end_date, last_report_date, report_date, con_date, sw_type).get_all()
    # sw_type = 3
    # select_industry = ['钴', '锂', '铝', '锂电池', '电池化学品', '硅料硅片', '光伏电池组件', '逆变器', '航空装备', '军工电子', '电动乘用车', '车身附件及饰件', '底盘与发动机系统', '空调', '冰洗', '酒店', '白酒', '数字芯片设计', '模拟芯片设计', '半导体设备', '医疗研发外包']
    # IndustryAnalysis(file_path, start_date, end_date, last_report_date, report_date, con_date, sw_type, select_industry).get_all()


"""
汇率和价格指数
"""
import pandas as pd
from datetime import datetime
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class RatesAndPrice:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_rates_and_price'

    def get_rates_and_price_data(self):
        """
        汇率和价格指数类数据：有效联邦基金利率、人民币兑美元汇率、美元指数、中国大宗商品价格指数、中国:进出口金额:当月值、
        在岸人民币汇率、离岸人民币汇率
        """
        index_list = ['G1161292', 'M0000185', 'M0000271', 'S5042881', 'M0085847']
        name_dict = {'G1161292': "federal_funds_rate", 'M0000185': "exchange_rate_dollar",
                     'M0000271': "dollar_index", 'S5042881': "CCPI", "M0085847": "in_export_value"}

        res = w.edb(','.join(index_list), self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch rates and price data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns=name_dict, inplace=True)
        rates = data.copy()
        # USDCNY.IB-USDCNH.FX
        res = w.wsd("USDCNY.IB,USDCNH.FX", "close", self.start_date, self.end_date, "")
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch rates and price data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns=name_dict, inplace=True)

        rates = pd.merge(rates, data, on='trade_date', how='outer').sort_values(by='trade_date').rename(
            columns={"USDCNY.IB": "USDCNY_IB", "USDCNH.FX": "USDCNH_FX"})

        return rates

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_rates_and_price_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                    create table mac_rates_and_price(
                    id int auto_increment primary key,
                    trade_date date not null unique,
                    federal_funds_rate decimal(4, 2),
                    exchange_rate_dollar decimal(6, 4),
                    dollar_index decimal(7, 4),
                    CCPI decimal(6, 2),
                    in_export_value decimal(10, 2),
                    USDCNY_IB decimal(6, 4),
                    USDCNH_FX decimal(6, 4)) 
                """
            create_table(self.table_name, sql_script)
            data = self.get_rates_and_price_data()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    RatesAndPrice('2005-01-01', '2021-06-10', is_increment=0).get_construct_result()


# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.orm.fedb import FEDB
from hbshare.fe.xwq.analysis.orm.hbdb import HBDB
from hbshare.fe.xwq.analysis.orm.plot import plot
from hbshare.fe.xwq.analysis.utils.const_var import TimeDateFormat
from hbshare.fe.xwq.analysis.utils.timedelta_utils import TimeDateUtil
from datetime import datetime
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib.ticker import FuncFormatter
import seaborn as sns
sns.set_style('white', {'font.sans-serif': ['simhei', 'Arial']})
industry_theme_dic = {'银行': '大金融', '非银金融': '大金融', '房地产': '大金融',
                      '食品饮料': '消费', '家用电器': '消费', '医药生物': '消费', '社会服务': '消费', '农林牧渔': '消费', '商贸零售': '消费', '美容护理': '消费',
                      '通信': 'TMT', '计算机': 'TMT', '电子': 'TMT', '传媒': 'TMT', '国防军工': 'TMT',
                      '交通运输': '制造', '机械设备': '制造', '汽车': '制造', '纺织服饰': '制造', '轻工制造': '制造', '电力设备': '制造',
                      '钢铁': '周期', '有色金属': '周期', '建筑装饰': '周期', '建筑材料': '周期', '基础化工': '周期', '石油石化': '周期', '煤炭': '周期', '公用事业': '周期', '环保': '周期',
                      '综合': '其他'}


def to_percent(temp, position):
    return '%1.0f'%(100 * temp) + '%'

def to_percent_r2(temp, position):
    return '%0.01f'%(100 * temp) + '%'

def fund_overview_plot(date, pic_path):
    """
    基金分布画图
    """
    label_type = 'OVERVIEW'
    fund_overview = FEDB().read_data(date, label_type)
    fund_overview['FUND_TYPE'] = fund_overview['LABEL_NAME'].apply(lambda x: x.split('_')[0])
    fund_overview['OPEN_CLOSE'] = fund_overview['LABEL_NAME'].apply(lambda x: x.split('_')[1])
    fund_overview = fund_overview[fund_overview['OPEN_CLOSE'] == '开放式基金']
    fig, ax = plt.subplots(figsize=(6, 6))
    ax.pie(fund_overview['LABEL_VALUE'].values, labels=fund_overview['FUND_TYPE'].values, autopct='%0.2f%%', colors=['#D55659', '#8588B7', '#7D7D7E'])
    plt.title('基金类型分布')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '类型分布'))
    return

def fund_valuation_plot(date, pic_path):
    """
    估值分析画图
    """
    label_type = 'VALUATION'
    fund_valuation = FEDB().read_data(date, label_type)
    pe_valuation = fund_valuation[fund_valuation['LABEL_NAME'].str.slice(0, 2) == 'PE']
    pe_valuation = pe_valuation.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    pe_valuation = pe_valuation.apply(lambda x: x / x.sum(), axis=1)
    pe_valuation = pe_valuation.sort_index()
    pe_valuation.columns = ['PE>50（含负）', '0<PE<=30', '30<PE<=50']
    pe_valuation = pe_valuation[['0<PE<=30', '30<PE<=50', 'PE>50（含负）']]
    fig, ax = plt.subplots(figsize=(12, 6))
    ax.stackplot(pe_valuation.index.tolist(), pe_valuation.T.values.tolist(), colors=['#D55659', '#8588B7', '#7D7D7E'], labels=pe_valuation.columns.tolist())
    plt.legend(loc=2)
    plt.xticks(rotation=90)
    plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.title('估值分布（PE）')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '估值分布（PE）'))

    pb_valuation = fund_valuation[fund_valuation['LABEL_NAME'].str.slice(0, 2) == 'PB']
    pb_valuation = pb_valuation.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    pb_valuation = pb_valuation.apply(lambda x: x / x.sum(), axis=1)
    pb_valuation = pb_valuation.sort_index()
    pb_valuation.columns = ['PB>5（含负）', '0<PB<=5']
    pb_valuation = pb_valuation[['0<PB<=5', 'PB>5（含负）']]
    fig, ax = plt.subplots(figsize=(12, 6))
    ax.stackplot(pb_valuation.index.tolist(), pb_valuation.T.values.tolist(), colors=['#D55659', '#8588B7'], labels=pb_valuation.columns.tolist())
    plt.legend(loc=2)
    plt.xticks(rotation=90)
    plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.title('估值分布（PB）')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '估值分布（PB）'))

    label_type = 'VALUATION_DIFF'
    fund_valuation_diff = FEDB().read_data(date, label_type)
    fund_valuation_diff = fund_valuation_diff.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    fund_valuation_diff = fund_valuation_diff.sort_index()
    fig, ax = plt.subplots(figsize=(12, 6))
    ax.bar(range(len(fund_valuation_diff.index)), fund_valuation_diff['PE_平均估值差'], color='#C94649', tick_label=fund_valuation_diff.index)
    plt.xticks(rotation=90)
    plt.title('平均估值差（PE）')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '平均估值差（PE）'))

    label_type = 'VALUATION_PREMIUM'
    fund_valuation_premium = FEDB().read_data(date, label_type)
    pe_valuation_premium = fund_valuation_premium[fund_valuation_premium['LABEL_NAME'].str.slice(0, 2) == 'PE']
    pe_valuation_premium = pe_valuation_premium.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    pe_valuation_premium = pe_valuation_premium.sort_index()
    fig, ax = plt.subplots(figsize=(12, 6))
    ax.plot(pe_valuation_premium.index, pe_valuation_premium['PE_核心资产估值溢价'], '#F04950')
    plt.xticks(rotation=90)
    plt.title('核心资产估值溢价（PE）')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '核心资产估值溢价（PE）'))

    pb_valuation_premium = fund_valuation_premium[fund_valuation_premium['LABEL_NAME'].str.slice(0, 2) == 'PB']
    pb_valuation_premium = pb_valuation_premium.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    pb_valuation_premium = pb_valuation_premium.sort_index()
    fig, ax = plt.subplots(figsize=(12, 6))
    ax.plot(pe_valuation_premium.index, pb_valuation_premium['PB_核心资产估值溢价'], '#F04950')
    plt.xticks(rotation=90)
    plt.title('核心资产估值溢价（PB）')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '核心资产估值溢价（PB）'))
    return

def fund_sector_plot(date, pic_path):
    """
    板块分析画图
    """
    label_type = 'SECTOR'
    fund_sector = FEDB().read_data(date, label_type)
    fund_sector = fund_sector.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    fund_sector = fund_sector.apply(lambda x: x / x.sum(), axis=1)
    fund_sector = fund_sector.sort_index()
    fig, ax = plt.subplots(figsize=(12, 6))
    ax.stackplot(fund_sector.index.tolist(), fund_sector.T.values.tolist(), colors=['#D55659', '#E1777A', '#8588B7', '#626697', '#7D7D7E'], labels=fund_sector.columns.tolist())
    plt.legend(loc=2)
    plt.xticks(rotation=90)
    plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.title('板块分布')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '板块分布'))

    hsi = pd.read_excel('{0}hsi_pe.xlsx'.format(pic_path))
    hsi = hsi[['交易日期', '市盈率TTM']]
    hsi.columns = ['TRADE_DATE', '恒生指数PE_TTM']
    hsi['TRADE_DATE'] = hsi['TRADE_DATE'].apply(lambda x: str(x).replace('-', ''))
    hsi['YEAR'] = hsi['TRADE_DATE'].apply(lambda x: x[:4])
    hsi['MONTH'] = hsi['TRADE_DATE'].apply(lambda x: x[4:6])
    hsi['YEAR_MONTH'] = hsi['TRADE_DATE'].apply(lambda x: x[:6])
    hsi = hsi.sort_values('TRADE_DATE').drop_duplicates('YEAR_MONTH', keep='last')
    hsi = hsi[hsi['MONTH'].isin(['03', '06', '09', '12'])]
    hsi['MONTH_DAY'] = hsi['MONTH'].replace('03', '0331').replace('06', '0630').replace('09', '0930').replace('12', '1231')
    hsi['REPORT_DATE'] = hsi['YEAR'] + hsi['MONTH_DAY']
    hsi = hsi[['REPORT_DATE', '恒生指数PE_TTM']].set_index('REPORT_DATE')
    hk = fund_sector[['主板-香港']].rename(columns={'主板-香港': '港股配置比例'}).merge(hsi, left_index=True, right_index=True, how='left')
    hk = hk[hk['港股配置比例'] != 0]
    hk = hk.sort_index().reset_index().rename(columns={'REPORT_HISTORY_DATE': 'REPORT_DATE'})
    fig, ax1 = plt.subplots(figsize=(12, 6))
    ax2 = ax1.twinx()
    sns.lineplot(ax=ax1, x='REPORT_DATE', y='港股配置比例', data=hk, label='港股配置比例', color='#F04950')
    sns.lineplot(ax=ax2, x='REPORT_DATE', y='恒生指数PE_TTM', data=hk, label='恒生指数PE_TTM', color='#6268A2')
    ax1.yaxis.set_major_formatter(FuncFormatter(to_percent_r2))
    ax1.legend(loc=2)
    ax2.legend(loc=1)
    ax1.set_xticklabels(labels=hk['REPORT_DATE'].unique().tolist(), rotation=90)
    ax2.set_xticklabels(labels=hk['REPORT_DATE'].unique().tolist(), rotation=90)
    ax1.set_xlabel('')
    ax2.set_xlabel('')
    ax1.set_ylabel('')
    ax2.set_ylabel('')
    plt.title('港股配置比例及估值')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '港股配置比例与估值'))
    return

def fund_theme_plot(date, pic_path):
    """
    主题分析画图
    """
    label_type = 'THEME'
    fund_theme = FEDB().read_data(date, label_type)
    fund_theme = fund_theme[['REPORT_HISTORY_DATE', 'LABEL_NAME', 'LABEL_VALUE']]
    fund_theme = fund_theme.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    fund_theme = fund_theme.apply(lambda x: x / x.sum(), axis=1)
    fund_theme = fund_theme.sort_index()
    fund_theme = fund_theme[['大金融', '周期', '制造', 'TMT', '消费', '其他']]
    fund_theme_latest5 = fund_theme.iloc[-5:]
    fig, ax = plt.subplots(figsize=(12, 6))
    color_list = ['#F04950', '#6268A2', '#959595', '#333335', '#EE703F', '#7E4A9B']
    for i, theme in enumerate(fund_theme_latest5.columns):
        ax.plot(fund_theme_latest5.index, fund_theme_latest5[theme], color_list[i], label=theme)
    plt.legend(loc=2)
    plt.xticks(rotation=90)
    plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.title('主题统计')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '主题统计'))

    fig, ax = plt.subplots(figsize=(12, 6))
    ax.stackplot(fund_theme.index.tolist(), fund_theme.T.values.tolist(), colors=['#D55659', '#E1777A', '#8588B7', '#626697', '#7D7D7E', '#A7A7A8'], labels=fund_theme.columns.tolist())
    plt.legend(loc=2)
    plt.xticks(rotation=90)
    plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.title('主题分布')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '主题分布'))
    return

def fund_industry_plot(date, pic_path, n=2, industry_name_1st='食品饮料'):
    """
    行业分析画图
    """
    row_list = [-(i + 1) for i in range(int(n))]

    label_type = 'INDUSTRY_SW1'
    fund_industry_1st = FEDB().read_data(date, label_type)
    fund_industry_1st = fund_industry_1st.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    fund_industry_1st = fund_industry_1st / 100.0
    fund_industry_1st_disp = fund_industry_1st.iloc[row_list, :].T.sort_values(date, ascending=False)
    fund_industry_1st_disp = fund_industry_1st_disp[fund_industry_1st_disp[date] != 0]
    date_list = [sorted(list(fund_industry_1st_disp.columns))[len(sorted(list(fund_industry_1st_disp.columns))) - i - 1] for i in range(len(sorted(list(fund_industry_1st_disp.columns))))]
    data_list = []
    for t in date_list:
        data_date = fund_industry_1st_disp[[t]].rename(columns={t: 'VALUE'})
        data_date['DATE'] = t
        data_list.append(data_date)
    data = pd.concat(data_list).reset_index()
    color_list = ['#C94649'] if n == 1 else ['#C94649', '#8588B7'] if n == 2 else ['#C94649', '#8588B7', '#7D7D7E'] if n == 3 else ['#C94649', '#EEB2B4', '#8588B7', '#7D7D7E'] if n == 4 else ['#C94649', '#EEB2B4', '#8588B7', '#B4B6D1', '#7D7D7E']
    fig, ax = plt.subplots(figsize=(12, 6))
    sns.barplot(ax=ax, x='LABEL_NAME', y='VALUE', data=data, hue='DATE', hue_order=date_list, palette=color_list)
    plt.legend(loc=1)
    plt.xticks(rotation=90)
    plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.xlabel('')
    plt.ylabel('')
    plt.title('一级行业持仓统计')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '一级行业持仓统计'))

    mom_abs = pd.DataFrame(fund_industry_1st_disp[date_list[0]] - fund_industry_1st_disp[date_list[1]]).reset_index()
    mom_abs.columns = ['INDUSTRY_NAME', 'MOM_ABS']
    fig, ax = plt.subplots(figsize=(12, 6))
    sns.barplot(ax=ax, x='INDUSTRY_NAME', y='MOM_ABS', data=mom_abs, palette=[color_list[0]])
    plt.xticks(rotation=90)
    plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.xlabel('')
    plt.ylabel('')
    plt.title('一级行业持仓环比变化')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '一级行业持仓环比变化'))

    label_type = 'INDUSTRY_SW2'
    fund_industry_2nd = FEDB().read_data(date, label_type)
    fund_industry_2nd = fund_industry_2nd.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    fund_industry_2nd = fund_industry_2nd / 100.0
    fund_industry_2nd_disp = fund_industry_2nd.iloc[row_list, :].T.sort_values(date, ascending=False).iloc[:20]
    fund_industry_2nd_disp = fund_industry_2nd_disp[fund_industry_2nd_disp[date] != 0]
    date_list = [sorted(list(fund_industry_2nd_disp.columns))[len(sorted(list(fund_industry_2nd_disp.columns))) - i - 1] for i in range(len(sorted(list(fund_industry_2nd_disp.columns))))]
    data_list = []
    for t in date_list:
        data_date = fund_industry_2nd_disp[[t]].rename(columns={t: 'VALUE'})
        data_date['DATE'] = t
        data_list.append(data_date)
    data = pd.concat(data_list).reset_index()
    color_list = ['#C94649'] if n == 1 else ['#C94649', '#8588B7'] if n == 2 else ['#C94649', '#8588B7', '#7D7D7E'] if n == 3 else ['#C94649', '#EEB2B4', '#8588B7', '#7D7D7E'] if n == 4 else ['#C94649', '#EEB2B4', '#8588B7', '#B4B6D1', '#7D7D7E']
    fig, ax = plt.subplots(figsize=(12, 6))
    sns.barplot(ax=ax, x='LABEL_NAME', y='VALUE', data=data, hue='DATE', hue_order=date_list, palette=color_list)
    plt.legend(loc=1)
    plt.xticks(rotation=90)
    plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.xlabel('')
    plt.ylabel('')
    plt.title('二级行业持仓统计')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '二级行业持仓统计'))

    fund_industry_1st_single = fund_industry_1st[[industry_name_1st]].sort_index()
    fund_industry_1st_single = fund_industry_1st_single.iloc[[-5, -4, -3, -2, -1], :]
    fig, ax = plt.subplots(figsize=(12, 6))
    ax.bar(range(len(fund_industry_1st_single.index)), fund_industry_1st_single[industry_name_1st], color='#C94649', tick_label=fund_industry_1st_single.index)
    plt.xticks(rotation=90)
    plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.title('{0}行业持仓占比变化'.format(industry_name_1st))
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '{0}行业持仓占比变化'.format(industry_name_1st)))
    return

def fund_market_value_plot(date, pic_path):
    """
    市值分析画图
    """
    label_type = 'MARKET_VALUE_1'
    fund_mv1 = FEDB().read_data(date, label_type)
    fund_mv1 = fund_mv1.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    fund_mv1 = fund_mv1[['HS300', 'ZZ500', 'ZZ1000', '非成分股']]
    fund_mv1.columns = ['沪深300', '中证500', '中证1000', '非成分股']
    fund_mv1 = fund_mv1.apply(lambda x: x / x.sum(), axis=1)
    fund_mv1_latest = fund_mv1.iloc[-6:]
    fig, ax1 = plt.subplots(figsize=(6, 6))
    ax2 = ax1.twinx()
    ax1.plot(fund_mv1_latest.index, fund_mv1_latest['沪深300'], color='#F04950', label='沪深300')
    ax2.plot(fund_mv1_latest.index, fund_mv1_latest['中证500'], color='#6268A2', label='中证500')
    ax2.plot(fund_mv1_latest.index, fund_mv1_latest['中证1000'], color='#959595', label='中证1000')
    # ax.stackplot(fund_mv1_latest.index.tolist(), fund_mv1_latest.T.values.tolist(), colors=['#D55659', '#E1777A', '#8588B7', '#7D7D7E'], labels=fund_mv1_latest5.columns.tolist())
    ax1.legend(loc=2)
    ax2.legend(loc=1)
    ax1.set_ylim([0.4, 0.6])
    ax2.set_ylim([0.1, 0.2])
    ax1.set_xticklabels(labels=fund_mv1_latest.index, rotation=90)
    ax2.set_xticklabels(labels=fund_mv1_latest.index, rotation=90)
    plt.xticks(rotation=90)
    ax1.yaxis.set_major_formatter(FuncFormatter(to_percent))
    ax2.yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.title('成分股占比变化')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '成分股占比变化'))

    label_type = 'MARKET_VALUE_2'
    fund_mv2 = FEDB().read_data(date, label_type)
    fund_mv2 = fund_mv2.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    fund_mv2 = fund_mv2[['SZ50', 'ZZ100', 'SZ180', 'HS300', 'TOTAL']]
    fund_mv2.columns = ['上证50', '中证100', '上证180', '沪深300', '总计']
    fund_mv2 = fund_mv2.apply(lambda x: x / x.iloc[-1], axis=1)
    fund_mv2_latest = fund_mv2.iloc[-6:, :-1]
    fig, ax = plt.subplots(figsize=(6, 6))
    color_list = ['#F04950', '#6268A2', '#959595', '#333335']
    for i, index in enumerate(fund_mv2_latest.columns):
        ax.plot(fund_mv2_latest.index, fund_mv2_latest[index], color_list[i], label=index)
    plt.legend(loc=2)
    plt.xticks(rotation=90)
    plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.title('权重股占比变化')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '权重股占比变化'))
    return

def fund_style_plot(date, pic_path):
    """
    风格分析画图
    """
    label_type = 'STYLE_1'
    fund_style1 = FEDB().read_data(date, label_type)
    fund_style1 = fund_style1.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    fund_style1 = fund_style1.drop('TOTAL', axis=1)
    fund_style1 = fund_style1.apply(lambda x: x / x.sum(), axis=1)
    fund_style1_latest = fund_style1[['成长', '平衡', '价值']].iloc[-6:]
    fig, ax1 = plt.subplots(figsize=(6, 6))
    ax2 = ax1.twinx()
    ax1.plot(fund_style1_latest.index, fund_style1_latest['成长'], color='#F04950', label='成长')
    ax2.plot(fund_style1_latest.index, fund_style1_latest['价值'], color='#6268A2', label='价值')
    ax1.legend(loc=2)
    ax2.legend(loc=1)
    ax1.set_ylim([0.50, 0.85])
    ax2.set_ylim([0.15, 0.40])
    ax1.set_xticklabels(labels=fund_style1_latest.index, rotation=90)
    ax2.set_xticklabels(labels=fund_style1_latest.index, rotation=90)
    plt.xticks(rotation=90)
    ax1.yaxis.set_major_formatter(FuncFormatter(to_percent))
    ax2.yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.title('风格统计')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '风格统计'))

    label_type = 'STYLE_2'
    fund_style2 = FEDB().read_data(date, label_type)
    fund_style2 = fund_style2.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    fund_style2 = fund_style2.drop('TOTAL', axis=1)
    fund_style2 = fund_style2.apply(lambda x: x / x.sum(), axis=1)
    fund_style2_latest = fund_style2[['大盘成长', '大盘平衡', '大盘价值', '中盘成长', '中盘平衡', '中盘价值', '小盘成长', '小盘平衡', '小盘价值']].iloc[-6:]
    fig, ax1 = plt.subplots(figsize=(6, 6))
    ax2 = ax1.twinx()
    ax1.plot(fund_style2_latest.index, fund_style2_latest['大盘成长'], color='#F04950', label='大盘成长')
    color_list = ['#6268A2', '#959595', '#333335', '#EE703F', '#7E4A9B', '#8A662C', '#44488E', '#BA67E9']
    for i, style in enumerate(fund_style2_latest.columns[1:]):
        ax2.plot(fund_style2_latest.index, fund_style2_latest[style], color_list[i], label=style)
    ax1.legend(loc=2)
    ax2.legend(loc=1)
    ax1.set_ylim([0.4, 0.65])
    ax2.set_ylim([0.0, 0.3])
    ax1.set_xticklabels(labels=fund_style2_latest.index, rotation=90)
    ax2.set_xticklabels(labels=fund_style2_latest.index, rotation=90)
    plt.xticks(rotation=90)
    ax1.yaxis.set_major_formatter(FuncFormatter(to_percent))
    ax2.yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.title('细分风格统计')
    plt.tight_layout()
    plt.savefig('{0}{1}'.format(pic_path, '细分风格统计'))
    return

def fund_barra(date, pic_path):
    label_type = 'BARRA'
    fund_barra = FEDB().read_data(date, label_type)
    fund_barra = fund_barra.pivot(index='REPORT_HISTORY_DATE', columns='LABEL_NAME', values='LABEL_VALUE')
    fund_barra.columns = ['Beta', '价值', '盈利', '成长', '杠杆率', '流动性', '动量', '波动率', '市值', '中市值']
    fund_barra = fund_barra.iloc[-2:].T
    fund_barra['变化'] = (fund_barra.iloc[:, -1] - fund_barra.iloc[:, -2])
    fund_barra.to_excel('{0}fund_barra.xlsx'.format(pic_path))
    return

def fund_holding(date, pic_path):
    label_type = 'STOCK'
    fund_stock = FEDB().read_data(date, label_type)
    date_list = sorted(fund_stock['REPORT_HISTORY_DATE'].unique().tolist())
    fund_stock = fund_stock[['REPORT_HISTORY_DATE', 'LABEL_NAME', 'LABEL_VALUE', 'LABEL_VALUE_STRING']]
    fund_stock['LABEL_NAME'] = fund_stock['LABEL_NAME'].apply(lambda x: x.split('_')[0])
    fund_stock['RANK'] = fund_stock[['REPORT_HISTORY_DATE', 'LABEL_NAME', 'LABEL_VALUE']].groupby(['REPORT_HISTORY_DATE', 'LABEL_NAME']).rank(method='first', ascending=False)
    fund_stock_list = []
    for td in date_list[-6:]:
        fund_stock_date = fund_stock[fund_stock['REPORT_HISTORY_DATE'] == td].sort_values(['LABEL_NAME', 'RANK'])
        fund_stock_date['一级行业'] = fund_stock_date.apply(lambda x: x['LABEL_NAME'] + '_' + str(int(x['RANK'])), axis=1)
        fund_stock_date = fund_stock_date.set_index('一级行业')
        fund_stock_date = fund_stock_date[['LABEL_VALUE_STRING']].rename(columns={'LABEL_VALUE_STRING': td})
        fund_stock_list.append(fund_stock_date)
    fund_stock = pd.concat(fund_stock_list, axis=1)
    fund_stock = fund_stock[date_list[::-1][:6]]
    fund_stock = fund_stock.reset_index()
    fund_stock['排名'] = fund_stock['一级行业'].apply(lambda x: x.split('_')[1])
    fund_stock['一级行业'] = fund_stock['一级行业'].apply(lambda x: x.split('_')[0])
    fund_stock['主题'] = fund_stock['一级行业'].apply(lambda x: industry_theme_dic[x])
    fund_stock['主题'] = fund_stock['主题'].astype('category')
    fund_stock['主题'].cat.reorder_categories(['制造', '消费', '周期', 'TMT', '大金融', '其他'], inplace=True)
    fund_stock = fund_stock.sort_values(['主题','一级行业', '排名'], ascending=[True, True, True])
    fund_stock.to_excel('{0}fund_stock.xlsx'.format(pic_path))
    return

def holding_analysis_plot(date, pic_path):
    """
    公募基金持仓分析画图
    """
    fund_overview_plot(date, pic_path)
    fund_valuation_plot(date, pic_path)
    fund_sector_plot(date, pic_path)
    fund_theme_plot(date, pic_path)
    fund_industry_plot(date, pic_path)
    fund_market_value_plot(date, pic_path)
    fund_style_plot(date, pic_path)
    fund_barra(date, pic_path)
    fund_holding(date, pic_path)
    return


if __name__ == "__main__":
    date = '20220630'
    pic_path = 'D:/Git/hbshare/hbshare/fe/xwq/data/mutual_analysis/'
    holding_analysis_plot(date, pic_path)





#!/usr/bin/python
#coding:utf-8
from __future__ import division
import time
from datetime import datetime
import pandas as pd
from hbshare.db.simu import cons as ct
import hbshare as hbs

"""
@author: Meng.lv
@contact: meng.lv@howbuy.com
@software: PyCharm
@file: nav.py.py
@time: 2020/9/18 9:18
"""


def get_simu_nav_by_code(code, start_date, end_date=datetime.now().strftime('%Y%m%d') , size=100, page=1, retry_count=3, pause=0.01, timeout=10):
    """
        根据基金代码查询历史净值、累计净值、复权单位净值(分页需要单独处理)
    :param code:
    :return:
    """
    api = hbs.hb_api()
    for _ in range(retry_count):
        time.sleep(pause)
        ct._write_console()

        url = ct.HOWBUY_SIMU_GET_NAV_LIST_BY_CODE % (ct.P_TYPE['https'], ct.DOMAINS['hbcgi'], code, start_date, end_date, size, page)

        org_js = api.query(url)
        status_code = int(org_js['common']['responseCode'])
        if status_code != 1:
            status = str(org_js['common']['responseContent'])
            raise ValueError(status)

        count = int(org_js['count'])

        if 'info' not in org_js:
            # status = "未查询到该基金数据"
            return pd.DataFrame([], columns=ct.HOWBUY_SIMU_NAV),0

        #['jzrq', 'jjjz', 'ljjz','fqdwjz', 'jjjzStr', 'ljjzStr']
        data = org_js['info']
        corp_df = pd.DataFrame(data, columns=ct.HOWBUY_SIMU_NAV)
        corp_df['jzrq'] = corp_df['jzrq'].astype(str)
        corp_df['jjjz'] = corp_df['jjjzStr'].str.replace("*", "").astype(float)
        corp_df['ljjz'] = corp_df['ljjzStr'].str.replace("*", "").astype(float)
        corp_df['fqdwjz'] = corp_df['fqdwjz'].str.replace("*", "").astype(float)
        corp_df['jzdw'] = corp_df['ljjz'].astype(float) / corp_df['jjjzStr'].str.replace("*", "").astype(float)

        corp_df = corp_df.drop('jjjzStr', axis=1)
        corp_df = corp_df.drop('ljjzStr', axis=1)
        return corp_df, count
    raise IOError(ct.NETWORK_URL_ERROR_MSG)


import pandas as pd
import numpy as np
from hbshare.quant.CChen.load_data import load_calendar
from hbshare.quant.CChen.fund_perf import performance_analysis
from hbshare.quant.CChen.cta_factor.const import composite_factor
from pyecharts.charts import Line, Grid
import pyecharts.options as opts
from pyecharts.components import Table
from pyecharts.options import ComponentTitleOpts


def load_factors(factor_dict, start_date, end_date, table, path, freq='w'):
    cal = load_calendar(start_date=start_date, end_date=end_date, freq=freq, db_path=path)

    data_all = pd.read_sql_query(
        'select TDATE, CLOSE, FACTOR from ' + table
        + ' where TDATE<=' + end_date.strftime('%Y%m%d')
        + ' and TDATE>=' + start_date.strftime('%Y%m%d')
        + ' and FACTOR in ' + str(tuple(factor_dict.keys()))
        + ' order by TDATE',
        path
    )
    data_all['FACTOR'] = data_all['FACTOR'].apply(lambda x: factor_dict[x])
    data_all = data_all.pivot(index='TDATE', columns='FACTOR', values='CLOSE')
    # data_all = data_all / data_all.loc[data_all.index[0], :]

    data = cal.merge(data_all.reset_index().rename(columns={'TDATE': 't_date'}), on='t_date', how='left')
    data = data.set_index(data['t_date'])[list(factor_dict.values())]
    data = data / data.loc[data.index[0], :]
    return data


def plot_lines(data, width=700, height=500, title='', legend_type='plain'):

    web = Line(
        init_opts=opts.InitOpts(
            # page_title='CTA大类因子',
            # width=str(width) + 'px',
            # height=str(height) + 'px',
        )
    ).add_xaxis(
        xaxis_data=data.index.tolist()
    )
    for i in data.columns:
        web.add_yaxis(
            series_name=i,
            y_axis=(data[i] / data[i][0]).round(4).tolist(),
            symbol="emptyCircle",
            is_symbol_show=True,
            label_opts=opts.LabelOpts(is_show=False),
            is_selected=True
        )

    web.set_global_opts(
        title_opts=opts.TitleOpts(
            title=title
        ),
        # tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        # toolbox_opts=opts.ToolboxOpts(is_show=True),
        xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False),
        yaxis_opts=opts.AxisOpts(
            axislabel_opts=opts.LabelOpts(formatter="{value}"),
            min_='dataMin',
            max_='dataMax',
        ),
        datazoom_opts=[
            opts.DataZoomOpts(range_start=0, range_end=100),
            # opts.DataZoomOpts(pos_left="5%", xaxis_index=0),
            # opts.DataZoomOpts(pos_right="5%", xaxis_index=1),
            opts.DataZoomOpts(type_="inside")
        ],
        legend_opts=opts.LegendOpts(
            type_=legend_type,
            pos_top="5%",
            pos_left='5%',
            pos_right='5%'
        )
    )

    # return web
    grid = (
        Grid(init_opts=opts.InitOpts(
            width=str(width) + "px", height=str(height) + "px")
        ).add(
            web, grid_opts=opts.GridOpts(
                # pos_right=str(chart_pos_right) + "%",
                # pos_left=str(chart_pos_left) + "%",
                pos_top='20%',
            ),
            is_control_axis_index=True
        )
    )
    return grid


def factor_holding_period(factor_dict, start_date, end_date, path, table='cta_factor'):
    # holding_long_list = []
    # holding_short_list = []
    holding_list = []
    factor_selected = []
    for i in factor_dict:
        # print(i)
        factor_selected.append(i)
        if i in composite_factor:
            holding_p = 0
            for b in composite_factor[i]:
                factor_data = pd.read_sql_query(
                    'select * from ' + table
                    + ' where TDATE<=' + end_date.strftime('%Y%m%d')
                    + ' and TDATE>=' + start_date.strftime('%Y%m%d')
                    + ' and FACTOR=\'' + b + '\'',
                    path
                )
                factor_data['UCODE'] = factor_data['EXCHANGE'].astype(str) + factor_data['PCODE'].astype(str)

                factor_pos = factor_data.pivot(index='TDATE', columns='UCODE', values='POS')

                pos_long = factor_pos[factor_pos > 0].fillna(0)
                pos_short = factor_pos[factor_pos < 0].fillna(0) * -1

                pos_long_cumdays = pos_long.cumsum() * pos_long * (pos_long - pos_long.shift(-1))
                pos_short_cumdays = pos_short.cumsum() * pos_short * (pos_short - pos_short.shift(-1))

                pos_long_days = (
                        pos_long_cumdays[pos_long_cumdays > 0].ffill().fillna(0)
                        - pos_long_cumdays[pos_long_cumdays > 0].ffill().fillna(0).shift(1)
                )
                pos_short_days = (
                        pos_short_cumdays[pos_short_cumdays > 0].ffill().fillna(0)
                        - pos_short_cumdays[pos_short_cumdays > 0].ffill().fillna(0).shift(1)
                )
                holding_period_all = np.nanmean(
                    pos_long_days[pos_long_days > 0].append(pos_short_days[pos_short_days > 0])
                )
                holding_p += holding_period_all
            holding_list.append(holding_p / len(composite_factor[i]))
        else:
            factor_data = pd.read_sql_query(
                'select * from ' + table
                + ' where TDATE<=' + end_date.strftime('%Y%m%d')
                + ' and TDATE>=' + start_date.strftime('%Y%m%d')
                + ' and FACTOR=\'' + i + '\'',
                path
            )
            factor_data['UCODE'] = factor_data['EXCHANGE'].astype(str) + factor_data['PCODE'].astype(str)

            factor_pos = factor_data.pivot(index='TDATE', columns='UCODE', values='POS')

            pos_long = factor_pos[factor_pos > 0].fillna(0)
            pos_short = factor_pos[factor_pos < 0].fillna(0) * -1

            pos_long_cumdays = pos_long.cumsum() * pos_long * (pos_long - pos_long.shift(-1))
            pos_short_cumdays = pos_short.cumsum() * pos_short * (pos_short - pos_short.shift(-1))

            pos_long_days = (
                    pos_long_cumdays[pos_long_cumdays > 0].ffill().fillna(0)
                    - pos_long_cumdays[pos_long_cumdays > 0].ffill().fillna(0).shift(1)
            )
            pos_short_days = (
                    pos_short_cumdays[pos_short_cumdays > 0].ffill().fillna(0)
                    - pos_short_cumdays[pos_short_cumdays > 0].ffill().fillna(0).shift(1)
            )
            # holding_period_long = np.nanmean(pos_long_days[pos_long_days > 0])
            # holding_period_short = np.nanmean(pos_short_days[pos_short_days > 0])
            holding_period_all = np.nanmean(pos_long_days[pos_long_days > 0].append(pos_short_days[pos_short_days > 0]))

            # holding_long_list.append(holding_period_long)
            # holding_short_list.append(holding_period_short)
            holding_list.append(holding_period_all)

    rrr = pd.DataFrame(
        {
            'index': factor_selected,
            # '多头持仓天数': holding_long_list,
            # '空头持仓天数': holding_short_list,
            '平均持仓天数': holding_list
        }
    )

    return rrr.set_index('index').round(2)


def factor_perf(factor_dict, start_date, end_date, path, table='cta_index'):
    index_data = pd.read_sql_query(
        'select * from ' + table
        + ' where TDATE<=' + end_date.strftime('%Y%m%d')
        + ' and TDATE>=' + start_date.strftime('%Y%m%d')
        + ' and FACTOR in ' + str(tuple(factor_dict.keys())),
        path
    ).pivot(index='TDATE', columns='FACTOR', values='CLOSE').reset_index().rename(columns={'TDATE': 't_date'})

    rrr = performance_analysis(
        data_df=index_data,
        start_date=start_date,
        end_date=end_date,
        ret_num_per_year=250,
        print_info=False
    )
    rr0 = rrr[0].iloc[[0, 1, 4]].set_index('index').T.rename(columns={
        start_date.strftime('%Y%m%d') + '以来累计': '累计收益率(%)',
        start_date.strftime('%Y%m%d') + '以来年化': '年化收益率(%)',
        '最大回撤': '最大回撤(%)',
    })
    rr0 = (rr0 * 100).round(2)
    return rr0


def factor_stats(factor_dict, start_date, end_date, path):
    fhp = factor_holding_period(factor_dict=factor_dict, start_date=start_date, end_date=end_date, path=path)
    fp = factor_perf(factor_dict=factor_dict, start_date=start_date, end_date=end_date, path=path)
    fff = fhp.reset_index().merge(fp.reset_index(), on='index')
    fff['因子代码'] = fff['index']
    fff['index'] = fff['index'].apply(lambda x: factor_dict[x])
    return fff


def plot_table(data, title='', subtitle=''):
    table = Table()
    headers = data.columns.tolist()
    rows = data.values.tolist()
    table.add(headers, rows)
    table.set_global_opts(
        title_opts=ComponentTitleOpts(title=title, subtitle=subtitle)
    )
    return table



"""
股票市场宏观类指标
"""
import pandas as pd
from datetime import datetime
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class StockValuation:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name1 = 'mac_stock_pe_ttm'
        self.table_name2 = 'mac_stock_pe_est'
        self.table_name3 = 'mac_stock_eps_ttm'

    def get_stock_market_data(self):
        """
        股票市场估值类数据：包含上证50、沪深300、中证500、中证1000、上证指数的PE_TTM和预测PE数据,和EPS_TTM数据
        """
        index_list = ['000001.SH', '000016.SH', '000300.SH', '000905.SH', '000852.SH']
        # PE_TTM
        res = w.wsd(','.join(index_list), 'pe_ttm', self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch pe_ttm data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        pe_ttm = data.copy()
        # pe_est
        data['year'] = data['trade_date'].apply(lambda x: datetime.strptime(x, '%Y%m%d').year)
        a, b = data.groupby('year')['trade_date'].min(), data.groupby('year')['trade_date'].max()
        interval_df = pd.merge(a.to_frame('start'), b.to_frame('end'), left_index=True, right_index=True)
        pe_est = []
        for year in interval_df.index:
            start_date, end_date = interval_df.loc[year, 'start'], interval_df.loc[year, 'end']
            res = w.wsd(','.join(index_list), 'pe_est', start_date, end_date, "year={}".format(year))
            if res.ErrorCode != 0:
                data = pd.DataFrame()
                print("fetch pe_est data error: start_date = {}, end_date = {}".format(start_date, end_date))
            else:
                if len(res.Data) == 1:
                    data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
                else:
                    data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
                data.index.name = 'trade_date'
                data.reset_index(inplace=True)
                data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            pe_est.append(data)
        pe_est = pd.concat(pe_est)
        # EPS_TTM
        res = w.wsd(','.join(index_list), 'eps_ttm', self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch eps_ttm data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        eps_ttm = data.copy()

        pe_ttm.rename(columns={"000001.SH": "SZZS", "000016.SH": "SZ50", "000300.SH": "HS300",
                               "000905.SH": "ZZ500", "000852.SH": "ZZ1000"}, inplace=True)
        pe_est.rename(columns={"000001.SH": "SZZS", "000016.SH": "SZ50", "000300.SH": "HS300",
                               "000905.SH": "ZZ500", "000852.SH": "ZZ1000"}, inplace=True)
        eps_ttm.rename(columns={"000001.SH": "SZZS", "000016.SH": "SZ50", "000300.SH": "HS300",
                                "000905.SH": "ZZ500", "000852.SH": "ZZ1000"}, inplace=True)

        return pe_ttm, pe_est, eps_ttm

    def get_construct_result(self):
        if self.is_increment == 1:
            pe_ttm, pe_est, eps_ttm = self.get_stock_market_data()
            # pe_ttm
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name1, ','.join(pe_ttm['trade_date'].tolist()))
            delete_duplicate_records(sql_script)
            WriteToDB().write_to_db(pe_ttm, self.table_name1)
            # pe_est
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name2, ','.join(pe_est['trade_date'].tolist()))
            delete_duplicate_records(sql_script)
            WriteToDB().write_to_db(pe_est, self.table_name2)
            # eps_ttm
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name3, ','.join(pe_ttm['trade_date'].tolist()))
            delete_duplicate_records(sql_script)
            WriteToDB().write_to_db(eps_ttm, self.table_name3)
        else:
            pe_ttm, pe_est, eps_ttm = self.get_stock_market_data()
            # pe_ttm
            sql_script = """
                    create table mac_stock_pe_ttm(
                    id int auto_increment primary key,
                    trade_date date not null unique,
                    SZZS decimal(5, 2),
                    SZ50 decimal(5, 2),
                    HS300 decimal(5, 2),
                    ZZ500 decimal(5, 2),
                    ZZ1000 decimal(5, 2)) 
                """
            create_table(self.table_name1, sql_script)
            WriteToDB().write_to_db(pe_ttm, self.table_name1)
            # pe_est
            sql_script = """
                    create table mac_stock_pe_est(
                    id int auto_increment primary key,
                    trade_date date not null unique,
                    SZZS decimal(5, 2),
                    SZ50 decimal(5, 2),
                    HS300 decimal(5, 2),
                    ZZ500 decimal(5, 2),
                    ZZ1000 decimal(5, 2)) 
                """
            create_table(self.table_name2, sql_script)
            WriteToDB().write_to_db(pe_est, self.table_name2)
            # eps_ttm
            sql_script = """
                    create table mac_stock_eps_ttm(
                    id int auto_increment primary key,
                    trade_date date not null unique,
                    SZZS decimal(5, 4),
                    SZ50 decimal(5, 4),
                    HS300 decimal(5, 4),
                    ZZ500 decimal(5, 4),
                    ZZ1000 decimal(5, 4)) 
                """
            create_table(self.table_name3, sql_script)
            WriteToDB().write_to_db(eps_ttm, self.table_name3)


class DividendRatio:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_stock_dividend'

    def get_stock_div_data(self):
        """
        指数股息率数据
        """
        index_list = ['000001.SH', '000016.SH', '000300.SH', '000905.SH', '000852.SH']
        # dividend
        res = w.wsd(','.join(index_list), 'dividendyield2', self.start_date, self.end_date, "")
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch dividend data error: start_date = {}, end_date = {}".format(self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))

        data.rename(columns={"000001.SH": "SZZS", "000016.SH": "SZ50", "000300.SH": "HS300",
                             "000905.SH": "ZZ500", "000852.SH": "ZZ1000"}, inplace=True)

        return data

    def get_construct_result(self):
        if self.is_increment == 1:
            dividend = self.get_stock_div_data()
            # dividend
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(dividend['trade_date'].tolist()))
            delete_duplicate_records(sql_script)
            WriteToDB().write_to_db(dividend, self.table_name)
        else:
            dividend = self.get_stock_div_data()
            # dividend
            sql_script = """
                    create table mac_stock_dividend(
                    id int auto_increment primary key,
                    trade_date date not null unique,
                    SZZS decimal(5, 4),
                    SZ50 decimal(5, 4),
                    HS300 decimal(5, 4),
                    ZZ500 decimal(5, 4),
                    ZZ1000 decimal(5, 4)) 
                """
            create_table(self.table_name, sql_script)
            WriteToDB().write_to_db(dividend, self.table_name)


if __name__ == '__main__':
    # StockValuation('2005-01-04', '2021-12-09', is_increment=1).get_construct_result()
    DividendRatio('2005-01-01', '2007-12-31', is_increment=1).get_construct_result()


# -*- coding: utf-8 -*-

from sqlalchemy import create_engine
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.sql.expression import Insert
import pandas as pd
import pymysql
import sqlalchemy
import sys
import time
import traceback
import warnings
warnings.filterwarnings('ignore', category=pymysql.Warning)


class FEDB:
    def __init__(self):
        self.fedb_engine = None
        self.fedb_connection = None
        self.create_connection()

    def create_connection(self):
        vaild = False
        while not vaild:
            try:
                self.fedb_engine = create_engine('mysql+pymysql://{0}:{1}@{2}:{3}/{4}'.format(
                                                 'admin', 'mysql', '192.168.223.152', '3306', 'fe_temp_data'),
                                   connect_args={'charset': 'utf8'}, pool_recycle=360,
                                   pool_size=2, max_overflow=10, pool_timeout=360)
                self.fedb_connection = self.fedb_engine.connect()
                vaild = True
            except Exception as e:
                self.close_connection()
                print('[FEDB] mysql connect error')
                time.sleep(5)
        return

    def close_connection(self):
        # 1) close connection
        try:
            if self.fedb_connection is not None:
                self.fedb_connection.close()
        except Exception as e:
            print('[FEDB] close connect error')
        # 2) dispose engine
        try:
            if self.fedb_engine is not None:
                self.fedb_engine.dispose()
        except Exception as e:
            print('[FEDB] dispose engine error')
        return

    def get_df(self, sql):
        valid = False
        df = None
        while not valid:
            try:
                df = pd.read_sql_query(sql, self.fedb_engine)
                self.close_connection()
                valid = True
            except Exception as e:
                self.close_connection()
                self.create_connection()
                print('[FEDB] sql error:{0}'.format(sql))
                time.sleep(5)
        return df

    def create_metadata(self, metadata):
        valid = False
        while not valid:
            try:
                metadata.create_all(self.fedb_engine)
                valid = True
            except Exception as e:
                self.close_connection()
                self.create_connection()
                exc_type, exc_value, exc_trackback = sys.exc_info()
                print(repr(traceback.format_exception(exc_type, exc_value, exc_trackback)))
                print('[FEDB] create metadata error')
            return metadata

    def insert_data(self, table, params):
        try:
            self.fedb_connection.execute(table.insert(), params)
        except Exception as e:
            conn = self.fedb_engine.connect()
            conn.execute(table.insert(), params)
            self.fedb_connection = conn
            exc_type, exc_value, exc_trackback = sys.exc_info()
            print(repr(traceback.format_exception(exc_type, exc_value, exc_trackback)))
            print('[FEDB] insert data error')
        return

    def replace_data(self, table, params):
        # @compiles(Insert)
        def replace_string(insert, compiler, **kw):
            s = compiler.visit_insert(insert, **kw)
            s = s.replace("INSERT INTO", "REPLACE INTO")
            return s

        valid = False
        times = 0
        while not valid and times < 5:
            try:
                times += 1
                self.fedb_connection.execute(table.insert(replace_string=""), params)
                valid = True
            except Exception as e:
                self.close_connection()
                self.create_connection()
                time.sleep(2)
                exc_type, exc_value, exc_trackback = sys.exc_info()
                print(repr(traceback.format_exception(exc_type, exc_value, exc_trackback)))
                print('[FEDB] replace data error')
            return

    def update_data(self, table, params):
        # @compiles(Insert)
        def append_string(insert, compiler, **kw):
            s = compiler.visit_insert(insert, **kw)
            if insert.kwards.get('on_duplicates_key_update'):
                fields = s[s.find("(") + 1:s.find(")")].replace(" ", "").split(",")
                generated_directive = ["{0}=VALUES({0})".format(field) for field in fields]
                return s + " ON DUPLICATES KEY UPDATE " + ",".join(generated_directive)
            return s

        valid = False
        times = 0
        while not valid and times < 5:
            try:
                times += 1
                self.fedb_connection.execute(table.insert(on_duplicates_key_update=True), params)
                valid = True
            except Exception as e:
                self.close_connection()
                self.create_connection()
                time.sleep(2)
                exc_type, exc_value, exc_trackback = sys.exc_info()
                print(repr(traceback.format_exception(exc_type, exc_value, exc_trackback)))
                print('[FEDB] update data error')
            return

    def insert_df(self, factor_df):
        table_name = 'mutual_fund_holding_label'
        metadata = sqlalchemy.MetaData()
        table = sqlalchemy.Table(
            table_name, metadata,
            sqlalchemy.Column('REPORT_DATE'),
            sqlalchemy.Column('REPORT_HISTORY_DATE'),
            sqlalchemy.Column('FUND_UNIVERSE'),
            sqlalchemy.Column('IS_ZC'),
            sqlalchemy.Column('LABEL_TYPE'),
            sqlalchemy.Column('LABEL_NAME'),
            sqlalchemy.Column('LABEL_VALUE'),
            sqlalchemy.Column('LABEL_VALUE_STRING')
        )
        metadata = self.create_metadata(metadata)
        params = []
        for idx in range(len(factor_df)):
            rows = factor_df.iloc[idx]
            param = dict()
            param['REPORT_DATE'] = rows['REPORT_DATE']
            param['REPORT_HISTORY_DATE'] = rows['REPORT_HISTORY_DATE']
            param['FUND_UNIVERSE'] = rows['FUND_UNIVERSE']
            param['IS_ZC'] = rows['IS_ZC']
            param['LABEL_TYPE'] = rows['LABEL_TYPE']
            param['LABEL_NAME'] = rows['LABEL_NAME']
            param['LABEL_VALUE'] = rows['LABEL_VALUE']
            param['LABEL_VALUE_STRING'] = rows['LABEL_VALUE_STRING']
            params.append(param)
            if len(params) > 1000:
                self.update_data(table, params)
                del params
                params = []
        if len(params) > 0:
            self.update_data(table, params)
        self.close_connection()
        return

    def insert_hbs_rank_his_df(self, factor_df):
        table_name = 'hbs_rank_his'
        metadata = sqlalchemy.MetaData()
        table = sqlalchemy.Table(
            table_name, metadata,
            sqlalchemy.Column('jjdm'),
            sqlalchemy.Column('jsrq'),
            sqlalchemy.Column('shift_ratio_size'),
            sqlalchemy.Column('c_level_size'),
            sqlalchemy.Column('shift_ratio_style'),
            sqlalchemy.Column('c_level_style'),
            sqlalchemy.Column('shift_ratio_theme'),
            sqlalchemy.Column('c_level_theme'),
            sqlalchemy.Column('shift_ratio_ind'),
            sqlalchemy.Column('c_level_ind'),
            sqlalchemy.Column('c_level_stock'),
        )
        metadata = self.create_metadata(metadata)
        params = []
        for idx in range(len(factor_df)):
            rows = factor_df.iloc[idx]
            param = dict()
            param['jjdm'] = rows['jjdm']
            param['jsrq'] = rows['jsrq']
            param['shift_ratio_size'] = rows['shift_ratio_size']
            param['c_level_size'] = rows['c_level_size']
            param['shift_ratio_style'] = rows['shift_ratio_style']
            param['c_level_style'] = rows['c_level_style']
            param['shift_ratio_theme'] = rows['shift_ratio_theme']
            param['c_level_theme'] = rows['c_level_theme']
            param['shift_ratio_ind'] = rows['shift_ratio_ind']
            param['c_level_ind'] = rows['c_level_ind']
            param['c_level_stock'] = rows['c_level_stock']
            params.append(param)
            if len(params) > 1000:
                self.update_data(table, params)
                del params
                params = []
        if len(params) > 0:
            self.update_data(table, params)
        self.close_connection()
        return

    def insert_fund_index(self, factor_df):
        table_name = 'fund_index'
        metadata = sqlalchemy.MetaData()
        table = sqlalchemy.Table(
            table_name, metadata,
            sqlalchemy.Column('TRADE_DATE'),
            sqlalchemy.Column('INDEX_SYMBOL'),
            sqlalchemy.Column('INDEX_NAME'),
            sqlalchemy.Column('INDEX_POINT'),
            sqlalchemy.Column('涨幅'),
            sqlalchemy.Column('近1月涨幅'),
            sqlalchemy.Column('近3月涨幅'),
            sqlalchemy.Column('近6月涨幅'),
            sqlalchemy.Column('近1年涨幅'),
            sqlalchemy.Column('今年以来涨幅'),
            sqlalchemy.Column('成立以来涨幅')
        )
        metadata = self.create_metadata(metadata)
        params = []
        for idx in range(len(factor_df)):
            rows = factor_df.iloc[idx]
            param = dict()
            param['TRADE_DATE'] = rows['TRADE_DATE']
            param['INDEX_SYMBOL'] = rows['INDEX_SYMBOL']
            param['INDEX_NAME'] = rows['INDEX_NAME']
            param['INDEX_POINT'] = rows['INDEX_POINT']
            param['涨幅'] = rows['涨幅']
            param['近1月涨幅'] = rows['近1月涨幅']
            param['近3月涨幅'] = rows['近3月涨幅']
            param['近6月涨幅'] = rows['近6月涨幅']
            param['近1年涨幅'] = rows['近1年涨幅']
            param['今年以来涨幅'] = rows['今年以来涨幅']
            param['成立以来涨幅'] = rows['成立以来涨幅']
            params.append(param)
            if len(params) > 1000:
                self.update_data(table, params)
                del params
                params = []
        if len(params) > 0:
            self.update_data(table, params)
        self.close_connection()
        return

    def read_fund_index_latest_date(self):
        sql = "SELECT max(TRADE_DATE) FROM fe_temp_data.fund_index GROUP BY INDEX_NAME"
        df = self.get_df(sql)
        latest_date = min(df['max(TRADE_DATE)'])
        return latest_date

    def read_fund_index_given_date(self, date):
        sql = "SELECT TRADE_DATE, INDEX_SYMBOL, INDEX_NAME, INDEX_POINT FROM fe_temp_data.fund_index WHERE TRADE_DATE = '{0}'".format(date)
        df = self.get_df(sql)
        return df

    def read_fund_index_gt_date(self, date):
        sql = "SELECT TRADE_DATE, INDEX_SYMBOL, INDEX_NAME, INDEX_POINT FROM fe_temp_data.fund_index WHERE TRADE_DATE >= '{0}'".format(date)
        df = self.get_df(sql)
        return df

    def read_data(self, date, label_type):
        sql = "SELECT REPORT_DATE, REPORT_HISTORY_DATE, FUND_UNIVERSE, IS_ZC, LABEL_TYPE, LABEL_NAME, LABEL_VALUE, LABEL_VALUE_STRING FROM fe_temp_data.mutual_fund_holding_label where REPORT_DATE='{0}' AND LABEL_TYPE='{1}' AND FUND_UNIVERSE='FOCUS_MUTUAL_FUND_UNIVERSE' AND IS_ZC=1".format(date, label_type)
        df = self.get_df(sql)
        df['REPORT_DATE'] = df['REPORT_DATE'].astype(str)
        df['REPORT_HISTORY_DATE'] = df['REPORT_HISTORY_DATE'].astype(str)
        df['IS_ZC'] = df['IS_ZC'].astype(int)
        return df

    def read_hbs_size_property_given_code(self, code):
        size_list = ['大盘', '中盘', '小盘']
        size = ','.join(size_list)
        size_rank_list = ['大盘_rank', '中盘_rank', '小盘_rank']
        size_rank = ','.join(size_rank_list)
        sql = "select jjdm, asofdate, shift_lv, cen_lv, {0}, shift_lv_rank, cen_lv_rank, {1} from fe_temp_data.hbs_size_property where jjdm='{2}' and asofdate=(SELECT max(asofdate) FROM fe_temp_data.hbs_size_property)".format(size, size_rank, code)
        df = self.get_df(sql)
        return df

    def read_size_exposure_given_code(self, code):
        sql = "select jjdm, jsrq, size_type, zjbl from fe_temp_data.hbs_size_exp where jjdm='{0}'".format(code)
        df = self.get_df(sql)
        return df

    def read_hbs_style_property_given_code(self, code):
        style_list = ['成长', '价值']
        style = ','.join(style_list)
        style_rank_list = ['成长_rank', '价值_rank']
        style_rank = ','.join(style_rank_list)
        sql = "select jjdm, asofdate, shift_lv, cen_lv, {0}, shift_lv_rank, cen_lv_rank, {1} from fe_temp_data.hbs_style_property where jjdm='{2}' and asofdate=(SELECT max(asofdate) FROM fe_temp_data.hbs_style_property)".format(style, style_rank, code)
        df = self.get_df(sql)
        return df

    def read_style_exposure_given_code(self, code):
        sql = "select jjdm, jsrq, style_type, zjbl from fe_temp_data.hbs_style_exp where jjdm='{0}'".format(code)
        df = self.get_df(sql)
        return df

    def read_hbs_size_style_property_history_given_code(self, code):
        sql = "select jjdm, jsrq, shift_ratio_size, c_level_size, shift_ratio_style, c_level_style from fe_temp_data.hbs_cen_shift_ratio_his_style where jjdm='{0}'".format(code)
        df = self.get_df(sql)
        return df

    def read_hbs_theme_industry_property_given_code(self, code):
        sql = "select jjdm, asofdate, ratio_theme, cen_theme, ratio_theme_rank, cen_theme_rank, ratio_ind_1 as ratio_ind, cen_ind_1 as cen_ind, ratio_ind_1_rank as ratio_ind_rank, cen_ind_1_rank as cen_ind_rank, industry_num, top5 from fe_temp_data.hbs_industry_property_new where jjdm='{0}' and asofdate=(SELECT max(asofdate) FROM fe_temp_data.hbs_industry_property_new)".format(code)
        df = self.get_df(sql)
        return df

    def read_hbs_theme_property_given_code(self, code):
        theme_list = ['大金融', '消费', 'TMT', '制造', '周期']
        theme = ','.join(theme_list)
        sql = "select jjdm, asofdate, {0} from fe_temp_data.jjpic_theme_p where jjdm='{1}' and asofdate=(SELECT max(asofdate) FROM fe_temp_data.jjpic_theme_p)".format(theme, code)
        df = self.get_df(sql)
        return df

    def read_hbs_industry_property_given_code(self, code):
        sql = "select jjdm, asofdate, yjxymc, zsbl_mean, zsbl_med, zsbl_mean_rank, zsbl_med_rank from fe_temp_data.hbs_industry_property_1_industry_level where jjdm='{0}' and asofdate=(SELECT max(asofdate) FROM fe_temp_data.hbs_industry_property_1_industry_level)".format(code)
        df = self.get_df(sql)
        return df

    def read_theme_exposure_given_code(self, code):
        theme_list = ['大金融', '消费', 'TMT', '制造', '周期']
        theme = ','.join(theme_list)
        sql = "select jjdm, jsrq, {0} from fe_temp_data.hbs_theme_exp where jjdm='{1}'".format(theme, code)
        df = self.get_df(sql)
        return df

    def read_industry_exposure_given_code(self, code):
        sql = "select jjdm, jsrq, yjxymc, zjbl from fe_temp_data.hbs_industry_class1_exp where jjdm='{0}'".format(code)
        df = self.get_df(sql)
        return df

    def read_hbs_theme_industry_property_history_given_code(self, code):
        sql = "select jjdm, jsrq, shift_ratio_theme, c_level_theme, shift_ratio as shift_ratio_ind, c_level as c_level_ind from fe_temp_data.hbs_cen_shift_ratio_his_industry1 where jjdm={0}".format(code)
        df = self.get_df(sql)
        return df

    def read_hbs_holding_property_given_code(self, code):
        sql = "select jjdm, asofdate, 个股集中度 from fe_temp_data.hbs_holding_property where jjdm='{0}' and asofdate=(SELECT max(asofdate) FROM fe_temp_data.hbs_holding_property)".format(code)
        df = self.get_df(sql)
        return df

    def read_hbs_holding_property_history_given_code(self, code):
        sql = "select jjdm, jsrq, cenlv as c_level_stock from fe_temp_data.hbs_stock_cenlv_his where jjdm='{0}'".format(code)
        df = self.get_df(sql)
        return df

    def read_hbs_stock_trading_property_given_code(self, code):
        sql = "select * from fe_temp_data.hbs_stock_trading_property where jjdm='{0}' and asofdate=(SELECT max(asofdate) FROM fe_temp_data.hbs_stock_trading_property)".format(code)
        df = self.get_df(sql)
        return df

    def read_hbs_rank_history_given_code(self, code):
        sql = "select jjdm, jsrq, shift_ratio_size, c_level_size, shift_ratio_style, c_level_style, shift_ratio_theme, c_level_theme, shift_ratio_ind, c_level_ind, c_level_stock from fe_temp_data.hbs_rank_his where jjdm='{0}'".format(code)
        df = self.get_df(sql)
        return df

    def read_hbs_his_data(self, table_name):
        sql = "select * from fe_temp_data.{0}".format(table_name)
        df = self.get_df(sql)
        return df

    def read_industry_fundamental(self, sw_type):
        sql = "select * from fe_temp_data.industry_fundamental where INDUSTRY_TYPE={0}".format(sw_type)
        df = self.get_df(sql)
        return df

    def read_industry_valuation(self, sw_type):
        sql = "select * from fe_temp_data.industry_valuation where INDUSTRY_TYPE={0}".format(sw_type)
        df = self.get_df(sql)
        return df

    def read_industry_data(self, cols, table_name, sw_type):
        cols = ','.join(cols)
        sql = "select {0} from fe_temp_data.{1} where INDUSTRY_TYPE={2}".format(cols, table_name, sw_type)
        df = self.get_df(sql)
        return df